Package: CIMICE
Type: Package
Title: CIMICE-R: (Markov) Chain Method to Inferr Cancer Evolution
Version: 1.20.0
Authors@R: 
    c(
        person("Nicolò", "Rossi",
            role=c("aut", "cre"), email="olocin.issor@gmail.com",
            comment = c("Lab. of Computational Biology and Bioinformatics, Department of Mathematics, Computer Science and Physics, University of Udine",ORCID = "0000-0002-6353-7396"))
    )
Description: 
    CIMICE is a tool in the field of tumor phylogenetics and
    its goal is to build a Markov Chain (called Cancer Progression Markov Chain,
    CPMC) in order to model tumor subtypes evolution.
    The input of CIMICE is a Mutational Matrix, so a boolean matrix representing
    altered genes in a collection of samples. These samples are assumed to be
    obtained with single-cell DNA analysis techniques and the tool is 
    specifically written to use the peculiarities of this data for the CMPC
    construction.
License: Artistic-2.0
Encoding: UTF-8
Imports: dplyr, ggplot2, glue, tidyr, igraph, networkD3, visNetwork,
        ggcorrplot, purrr, ggraph, stats, utils, maftools, assertthat,
        tidygraph, expm, Matrix
RoxygenNote: 7.1.2
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, testthat, webshot
biocViews: Software, BiologicalQuestion, NetworkInference,
        ResearchField, Phylogenetics, StatisticalMethod,
        GraphAndNetwork, Technology, SingleCell
BugReports: https://github.com/redsnic/CIMICE/issues
URL: https://github.com/redsnic/CIMICE
BiocType: Software
git_url: https://git.bioconductor.org/packages/CIMICE
git_branch: RELEASE_3_23
git_last_commit: 80dd32c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:14:39 UTC; biocbuild
Author: Nicolò Rossi [aut, cre] (Lab. of Computational Biology and
    Bioinformatics, Department of Mathematics, Computer Science and
    Physics, University of Udine, ORCID:
    <https://orcid.org/0000-0002-6353-7396>)
Maintainer: Nicolò Rossi <olocin.issor@gmail.com>

# Generated by roxygen2: do not edit by hand

export(annotate_mutational_matrix)
export(binary_radix_sort)
export(build_subset_graph)
export(build_topology_subset)
export(chunk_reader)
export(compact_dataset)
export(computeDWNW)
export(computeUPW)
export(compute_weights_default)
export(corrplot_from_mutational_matrix)
export(corrplot_genes)
export(corrplot_samples)
export(dataset_preprocessing)
export(dataset_preprocessing_population)
export(draw_ggraph)
export(draw_networkD3)
export(draw_visNetwork)
export(example_dataset)
export(example_dataset_withFreqs)
export(finalize_generator)
export(fix_clonal_genotype)
export(format_labels)
export(gene_mutations_hist)
export(get_no_of_children)
export(graph_non_transitive_subset_topology)
export(make_dataset)
export(make_generator_stub)
export(make_labels)
export(normalizeDWNW)
export(normalizeUPW)
export(perturb_dataset)
export(plot_generator)
export(prepare_generator_edge_set_command)
export(prepare_labels)
export(quick_run)
export(read)
export(read_CAPRI)
export(read_CAPRI_string)
export(read_CAPRIpop)
export(read_CAPRIpop_string)
export(read_MAF)
export(read_matrix)
export(remove_transitive_edges)
export(sample_mutations_hist)
export(select_genes_on_mutations)
export(select_samples_on_mutations)
export(set_generator_edges)
export(simulate_generator)
export(to_dot)
export(update_df)
import(Matrix)
importFrom(assertthat,are_equal)
importFrom(assertthat,assert_that)
importFrom(dplyr,"%>%")
importFrom(dplyr,count)
importFrom(dplyr,enexprs)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_all)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(expm,"%^%")
importFrom(ggcorrplot,ggcorrplot)
importFrom(ggplot2,aes)
importFrom(ggplot2,arrow)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,unit)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_edge_loop)
importFrom(ggraph,geom_node_label)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,ggraph)
importFrom(ggraph,label_rect)
importFrom(glue,glue)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as_adj)
importFrom(igraph,as_edgelist)
importFrom(igraph,as_ids)
importFrom(igraph,get.edge.ids)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_edgelist)
importFrom(igraph,head_of)
importFrom(igraph,tail_of)
importFrom(maftools,mutCountMatrix)
importFrom(maftools,read.maf)
importFrom(networkD3,forceNetwork)
importFrom(networkD3,igraph_to_networkD3)
importFrom(purrr,imap_chr)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map2_chr)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_int)
importFrom(purrr,rbernoulli)
importFrom(stats,cor)
importFrom(tidygraph,"%E>%")
importFrom(tidygraph,"%N>%")
importFrom(tidygraph,activate)
importFrom(tidygraph,as_tbl_graph)
importFrom(tidygraph,bind_edges)
importFrom(tidyr,drop_na)
importFrom(utils,read.csv)
importFrom(visNetwork,toVisNetworkData)
importFrom(visNetwork,visHierarchicalLayout)
importFrom(visNetwork,visNetwork)

# CIMICE-R

CIMICE is a tool in the field of tumor phylogenetics and
its goal is to build a Markov Chain (called Cancer Progression Markov Chain, CPMC) in order to model tumor subtypes evolution.
The input of CIMICE is a Mutational Matrix, so a boolean matrix representing altered genes in a
collection of samples. These samples are assumed to be obtained with single-cell DNA analysis techniques and
the tool is specifically written to use the peculiarities of this data for the CMPC construction.
See [this repository](https://github.com/redsnic/tumorEvolutionWithMarkovChains/tree/master/GenotypeEvolutionPaths) for the original Java version of this tool.

### Installation from GitHub:

```{R}
# install package
devtools::install_github("redsnic/CIMICE", build_vignettes = TRUE, dependencies = TRUE)
library(CIMICE)

# simple example
quick_run(example_dataset())
# show guide
browseVignettes("CIMICE")
```

remember to install [**pandoc**](https://pandoc.org/installing.html) to build the vignettes. 

### Input Formats:

#### CAPRI format

```
s/g    gene_1 gene_2 ... gene_n
sample_1 1 0 ... 0
...
sample_m 1 1 ... 1
```

#### CAPRIpop format

```
s/g    gene_1 gene_2 ... gene_n freq
sample_1 1 0 ... 0 freq_s1
...
sample_m 1 1 ... 1 freq_sm
```

(With this format, feature selection for most mutated samples or genes is disabled in the web application)

### Web Application:

A simple `shiny` web application to use CIMICE is available [here](https://redsnic.shinyapps.io/CIMICE_WEB/).

#' Filter dataset by genes' mutation count
#'
#' Dataset filtering on genes, based on their mutation count
#'
#' @param mutmatrix input dataset (mutational matrix) to be reduced
#' @param n number of genes to be kept
#' @param desc TRUE: select the n least mutated genes,
#' FALSE: select the n most mutated genes
#'
#' @return the modified dataset (mutational matrix)
#'
#' @examples
#'
#' # keep information on the 100 most mutated genes
#' select_genes_on_mutations(example_dataset(), 5)
#' # keep information on the 100 least mutated genes
#' select_genes_on_mutations(example_dataset(), 5, desc = FALSE)
#'
#' @export select_genes_on_mutations
select_genes_on_mutations <- function(mutmatrix, n, desc = TRUE){
    # transpose dataset to operate on genes
    temp.mutmatrix <- t(mutmatrix)
    # sort by sum
    sums <- rowSums(temp.mutmatrix, na.rm = TRUE)
    
    # sort and keep first n elements
    temp.mutmatrix <- temp.mutmatrix[order(sums, decreasing = desc),]
    temp.mutmatrix <- temp.mutmatrix[seq(1,min(n,nrow(temp.mutmatrix))),]
    
    # transpose back
    t(temp.mutmatrix)
}

#' Filter dataset by samples' mutation count
#'
#' Dataset filtering on samples, based on their mutation count
#'
#' @param mutmatrix input dataset (mutational matrix) to be reduced
#' @param n number of samples to be kept
#' @param desc T: select the n least mutated samples,
#' F: select the n most mutated samples
#'
#' @return the modified dataset (mutational matrix)
#'
#' @examples
#' require(dplyr)
#' # keep information on the 5 most mutated samples
#' select_samples_on_mutations(example_dataset(), 5)
#' # keep information on the 5 least mutated samples
#' select_samples_on_mutations(example_dataset(), 5, desc = FALSE)
#' # combine selections
#' select_samples_on_mutations(example_dataset() , 5, desc = FALSE) %>%
#'     select_genes_on_mutations(5)
#'
#' @export select_samples_on_mutations
select_samples_on_mutations <- function(mutmatrix, n, desc = TRUE){
    # sort by sum
    sums <- rowSums(mutmatrix, na.rm = TRUE)
    
    # sort and keep first n elements
    mutmatrix <- mutmatrix[order(sums, decreasing = desc),]
    mutmatrix <- mutmatrix[seq(1,min(n,nrow(mutmatrix))),]

    # transpose back
    mutmatrix
}

#' Radix sort for a binary matrix
#' 
#' Sort the rows of a binary matrix in ascending order
#' 
#' @param mat a binary matrix (of 0 and 1)
#' 
#' @return the sorted matrix
#' 
#' @examples 
#' require(Matrix)
#' m <- Matrix(c(1,1,0,1,0,0,0,1,1), sparse = TRUE, ncol = 3)
#' binary_radix_sort(m)
#' 
#' @export binary_radix_sort
binary_radix_sort <- function(mat){
    zeroes <- c()
    ones <- c()
    for(j in seq(ncol(mat), 1)){
        for(i in seq(1,nrow(mat))){
            if(mat[i,j] == 0){
                zeroes <- c(zeroes, i)
            }else{
                ones <- c(ones, i)
            }
        }
        mat <- mat[c(zeroes, ones), ]
        zeroes <- c()
        ones <- c()
    }
    mat
}


#' Compact dataset rows 
#'
#' Count duplicate rows
#' and compact the dataset (mutational). The column
#' 'freq' will contain the counts for each row.
#'
#' @param mutmatrix input dataset (mutational matrix)
#'
#' @return a list with matrix (the compacted dataset (mutational matrix)), counts 
#' (frequencies of genotypes) and row_names (comma separated string of sample IDs) fields
#' 
#' @examples
#' compact_dataset(example_dataset())
#'
#' @export compact_dataset
compact_dataset <- function(mutmatrix){
    mutmatrix <- binary_radix_sort(mutmatrix)
    # manage row names
    old_row_names <- rownames(mutmatrix)
    if(is.null(old_row_names)){
        old_row_names <- map_chr(seq(1,nrow(mutmatrix)), ~ as.character(.))
    }
    row_names <- list()
    cnt <- 1
    counts <- c()
    # keep only a line per genotype (the first line is always new)
    valid_indexes <- c(1)
    pos <- 1
    row_names[[pos]] <- old_row_names[1]
    for(i in seq(1,nrow(mutmatrix)-1)){
        if( all(mutmatrix[i,] == mutmatrix[i+1,]) ){
            cnt <- cnt + 1
            row_names[[pos]] <- paste(row_names[[pos]], old_row_names[i+1], sep=", ")
        }else{
            valid_indexes <- c(valid_indexes, i+1)
            counts <- c(counts, cnt)
            cnt <- 1
            pos <- pos + 1
            row_names[[pos]] <- old_row_names[i+1]
        }
    }
    counts <- c(counts, cnt) 
    list(matrix = mutmatrix[c(valid_indexes), ], counts = counts, row_names = row_names)
}


#' Compute subset relation as edge list
#'
#' Create an edge list E representing the
#' 'subset' relation for binary strings so that:
#' \deqn{ (A,B) in E <=> forall(i) : A[i] -> B[i] }
#'
#' @param samples input dataset (mutational matrix) as matrix
#'
#' @return the computed edge list
#'
#' @examples
#' require(dplyr)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' build_topology_subset(samples)
#'
#' @export build_topology_subset
build_topology_subset <- function(samples){
    # computing subset relation
    edges = list()
    index = 1
    # simple for loop that computes
    # the subset relation pairwisely
    for(i in seq(1,nrow(samples))){
        for(j in seq(1,nrow(samples))){
            if(i!=j){
                r1 = samples[i,]
                r2 = samples[j,]
                if( ! ((-1) %in% (r1 - r2)) ) {
                    edges[[index]] <- c(j,i)
                    index <- index + 1
                }
            }
        }
    }
    edges
}

#' Prepare node labels based on genotypes
#'
#' Prepare node labels so that each node is labelled with a
#' comma separated list of the alterated genes
#' representing its associated genotype.
#'
#' Note that after this procedure the user is
#' expected also to run fix_clonal_genotype
#' to also add the clonal genortype to the
#' mutational matrix if it is not present.
#'
#' @param samples input dataset (mutational matrix) as matrix
#' @param genes list of gene names (in the columns' order)
#'
#' @return the computed edge list
#'
#' @examples
#' require(dplyr) 
#' 
#' # compact
#' compactedDataset <- compact_dataset(example_dataset())
#' samples <- compactedDataset$matrix
#' 
#' # save genes' names
#' genes <- colnames(compactedDataset$matrix)
#' 
#' # keep the information on frequencies for further analysis
#' freqs <- compactedDataset$counts/sum(compactedDataset$counts)
#' 
#' # prepare node labels listing the mutated genes for each node
#' labels <- prepare_labels(samples, genes)
#'
#' @export prepare_labels
prepare_labels <- function(samples, genes){
    # prepare labels with the alterated genes accordingly to node's bit vector
    labels = apply(samples, MARGIN = 1, FUN = function(x) genes[x==1] )
    # concatenate the genes' names
    labels = lapply(labels,
                    function (x){
                        if (length(x) > 0)
                            paste(x, collapse=", ")
                        else
                            "Clonal"
                    }
    )
    labels
}

#' Manage Clonal genotype in data
#'
#' Fix the absence of the clonal genotype in the data (if needed)
#'
#' @param samples input dataset (mutational matrix) as matrix
#' @param freqs genotype frequencies (in the rows' order)
#' @param labels list of gene names (in the columns' order)
#' @param matching_samples list of sample names matching each genotype
#'
#' @return a named list containing the fixed "samples", "freqs" and "labels"
#'
#' @examples
#' require(dplyr) 
#' 
#' # compact
#' compactedDataset <- compact_dataset(example_dataset())
#' samples <- compactedDataset$matrix
#' 
#' # save genes' names
#' genes <- colnames(compactedDataset$matrix)
#' 
#' # keep the information on frequencies for further analysis
#' freqs <- compactedDataset$counts/sum(compactedDataset$counts)
#' 
#' # prepare node labels listing the mutated genes for each node
#' labels <- prepare_labels(samples, genes)
#' if( is.null(compactedDataset$row_names) ){
#'   compactedDataset$row_names <- rownames(compactedDataset$matrix)
#' }
#' matching_samples <- compactedDataset$row_names
#' # matching_samples
#' matching_samples 
#'
#' # fix Colonal genotype absence, if needed
#' fix <- fix_clonal_genotype(samples, freqs, labels, matching_samples)
#'
#' @export fix_clonal_genotype
fix_clonal_genotype <- function(samples, freqs, labels, matching_samples){
    # if no clonal genotype is found
    if (!(0 %in% rowSums(samples))){
        # add a 0 frequency genotype without mutations to the mutational matrix
        samples <- rbind(samples, map_dbl(seq(1,ncol(samples)), function(x) 0) )
        rownames(samples)[nrow(samples)] <- "Clonal" 
        freqs <- c(freqs,0)
        # update labels
        labels <- c(labels,"Clonal")
        matching_samples <- c(matching_samples, "Clonal")
    }
    list("labels" = labels, "samples" = samples, "freqs"=freqs, "matching_samples"=matching_samples)
}



#' Remove transitive edges and prepare graph
#'
#' Create a graph from the "build_topology_subset" edge list, so
#' that it respects the subset relation, omitting the transitive edges.
#'
#' @param edges edge list, built from "build_topology_subset"
#' @param labels list of node labels, to be paired with the graph
#'
#' @return a graph with the subset topology, omitting transitive edges
#'
#' @examples
#' require(dplyr)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' edges <- build_topology_subset(samples)
#' g <- build_subset_graph(edges, labels)
#'
#' @export build_subset_graph
build_subset_graph <- function(edges, labels){
    
    # {OLDER CODE, use if transitive reduction gets fixed}
    # prepare the actual graph
    #g <- graph_from_edgelist(t(simplify2array(edges)))
    # remove transitive edges using transitive.reduction from the "nem" package
    # g = graph_from_adjacency_matrix(transitive.reduction(as.matrix(as_adj(g))))
    # altenative approach with "relations" package
    #E <- transitive_reduction(
    #    endorelation(graph = as.list(data.frame(t(as_edgelist(g))))))
    #g <- graph_from_adjacency_matrix(E$.Data$incidence)
    # add labels to node
    #V(g)$label <- labels
    #g
    g <- edges %>% remove_transitive_edges %>% graph_from_edgelist
    V(g)$label <- labels
    g
}

#' Remove transitive edges from an edgelist
#'
#' Remove transitive edges from an edgelist. This procedure is temporary to
#' cover a bug in 'relations' package.
#'
#' @param E edge list, built from "build_topology_subset"
#'
#' @return a new edgelist without transitive edges (as a N*2 matrix)
#'
#' @examples
#' l <- list(c(1,2),c(2,3), c(1,3))
#' remove_transitive_edges(l)
#'
#' @export remove_transitive_edges
remove_transitive_edges <- function(E){
  
  discarded <- rep(0, times=length(E))
  stop <- FALSE # stop at first counterexample
  
  # for each edge A -> B seek a pair <A -> C, C -> B>
  for(i in seq(1,length(E))){
    for(e1 in E){
      for(e2 in E){
        if(e1[2] == e2[1] && E[[i]][1] == e1[1] && E[[i]][2] == e2[2]){
          discarded[i] <- 1
          stop <- TRUE
          break
        }
      }
      if(stop){
        stop <- FALSE
        break
      }
    }
  }
  
  # prepare output with non discarded edges
  out <- matrix(rep(0,times=2*length(which(discarded == 0))), ncol=2)
  j <- 1
  for(i in seq(1,length(E))){
    if(discarded[i] == 0){
      out[j,1] <- E[[i]][1]
      out[j,2] <- E[[i]][2]
      j <- j + 1 
    }
  }
  out
}

#' Get number of children
#'
#' Compute number of children for each node given an adj matrix
#'
#' @param A Adjacency matrix of the graph g
#' @param g a graph
#'
#' @return a vector containing the number of children for each node in g
#'
#' @examples
#' require(dplyr)
#' require(igraph)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' A <- as_adj(g)
#' get_no_of_children(A, g)
#'
#' @export get_no_of_children
get_no_of_children <- function(A,g){
    no.of.children <- numeric(length(V(g)))
    for( v in V(g) ){
        no.of.children[v] = sum(A[v,])
    }
    no.of.children
}

# This script contains functions to quicken the
# data analysis and to prepare working examples
utils::globalVariables(c("A","B","C","D","freq"))

#' Creates a simple example dataset
#'
#' @return a simple mutational matrix
#'
#' @examples
#' example_dataset()
#'
#' @export example_dataset
example_dataset <- function(){
    make_dataset(A,B,C,D) %>% # genes
        # samples
        update_df("S1", 0, 0, 0, 1) %>%
        update_df("S2", 1, 0, 0, 0) %>%
        update_df("S3", 1, 0, 0, 0) %>%
        update_df("S4", 1, 0, 0, 1) %>%
        update_df("S5", 1, 1, 0, 1) %>%
        update_df("S6", 1, 1, 0, 1) %>%
        update_df("S7", 1, 0, 1, 1) %>%
        update_df("S8", 1, 1, 0, 1)
}

#' Creates a simple example dataset with frequency column
#'
#' @return a simple mutational matrix
#'
#' @examples
#' example_dataset_withFreqs()
#'
#' @export example_dataset_withFreqs
example_dataset_withFreqs <- function(){
    dataset <- make_dataset(A,B,C,D) %>% # genes
        # samples
        update_df("G1", 0, 0, 0, 1) %>%
        update_df("G2", 1, 0, 0, 0) %>%
        update_df("G3", 1, 0, 0, 1) %>%
        update_df("G4", 1, 1, 0, 1) %>%
        update_df("G5", 1, 0, 1, 1) %>%
        update_df("G6", 1, 1, 0, 1)
    counts <- c(1,2,1,2,1,1)
    list(matrix = dataset, counts = counts)
}

#' Run CIMICE preprocessing
#'
#' executes the preprocessing steps of CIMICE
#'
#' Preprocessing steps:
#'
#' 1) dataset is compacted
#'
#' 2) genotype frequencies are computed
#'
#' 3) labels are prepared
#'
#' @param dataset a mutational matrix as a (sparse) matrix
#'
#' @return a list containing the mutational matrix ("samples"),
#' the mutational frequencies of the genotypes ("freqs"),
#' the node labels ("labels") and finally the gene names ("genes")
#'
#' @examples
#' require(dplyr)
#' example_dataset() %>% dataset_preprocessing
#'
#' @export dataset_preprocessing
dataset_preprocessing <- function(dataset){
    # compact
    compactedDataset <- compact_dataset(dataset)
    # run preprocessing on genotypes and frequencies
    dataset_preprocessing_population(compactedDataset)
}

#' Run CIMICE preprocessing for poulation format dataset
#'
#' executes the preprocessing steps of CIMICE
#'
#' Preprocessing steps:
#'
#' 1) genotype frequencies are computed
#'
#' 2) labels are prepared
#'
#' @param compactedDataset a list (matrix: a mutational matrix, 
#' counts: number of samples with given genotype). 
#' "counts" is normalized automatically. 
#'
#' @return a list containing the mutational matrix ("samples"),
#' the mutational frequencies of the genotypes ("freqs"),
#' the node labels ("labels") and finally the gene names ("genes")
#'
#' @examples
#' require(dplyr)
#' example_dataset_withFreqs() %>% dataset_preprocessing_population
#'
#' @export dataset_preprocessing_population
dataset_preprocessing_population <- function(compactedDataset){
    # dataset is already compacted by hypothesys
    # prepare for the analysis
    samples <- compactedDataset$matrix
    # save genes' names
    genes <- colnames(compactedDataset$matrix)
    # keep the information on frequencies for further analysis
    freqs <- compactedDataset$counts/sum(compactedDataset$counts)
    # prepare node labels listing the mutated genes for each node
    labels <- prepare_labels(samples, genes)
    # prepare node labels displaying genotype names
    if( is.null(compactedDataset$row_names) ){
        compactedDataset$row_names <- rownames(compactedDataset$matrix)
    }
    matching_samples <- compactedDataset$row_names
    # fix Colonal genotype absence, if needed
    fix <- fix_clonal_genotype(samples, freqs, labels, matching_samples)
    samples <- fix[["samples"]]
    freqs <- fix[["freqs"]]
    labels <- fix[["labels"]]
    matching_samples <- fix[["matching_samples"]]
    # return a list with the prepared dataset and its additional information
    list("samples" = samples, "freqs" = freqs,
        "labels" = labels, "genes" = genes, "matching_samples" = matching_samples)
}

#' Default preparation of graph topology
#'
#' By default, CIMICE computes the relation between
#' genotypes using the subset relation.
#' For the following steps it is also important
#' that the transitive edges are removed.
#'
#' @param samples mutational matrix
#' @param labels genotype labels
#'
#' @return a graph with the wanted topology
#'
#' @examples
#' require(dplyr)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' graph_non_transitive_subset_topology(samples, labels)
#' @export graph_non_transitive_subset_topology
graph_non_transitive_subset_topology <- function(samples, labels){
    # compute edges based on subset relation
    edges <- build_topology_subset(samples)
    # remove transitive edges and prepare igraph object
    build_subset_graph(edges, labels)
}

#' Compute default weights
#'
#' This procedure computes the weights for edges of a
#' graph accordingly to CIMICE specification.
#' (See vignettes for further explainations)
#'
#' @param g a graph (must be a DAG with no transitive edges)
#' @param freqs observed frequencies of genotypes
#'
#' @return a graph with the computed weights
#'
#' @examples
#' require(dplyr)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' compute_weights_default(g, freqs)
#'
#' @export compute_weights_default
compute_weights_default <- function(g, freqs){
    # prepare adj matrix
    A <- as_adj(g)
    # pre-compute exiting edges from each node
    no.of.children <- get_no_of_children(A,g)
    # compute the four steps
    upWeights <- computeUPW(g, freqs, no.of.children, A)
    normUpWeights <- normalizeUPW(g, freqs, no.of.children, A, upWeights)
    downWeights <- computeDWNW(g, freqs, no.of.children, A, normUpWeights)
    normDownWeights <- normalizeDWNW(g, freqs, no.of.children, A, downWeights)
    normDownWeights
}

#' Run CIMICE defaults
#'
#' This function executes CIMICE analysis on a dataset using default settings.
#'
#' @param dataset a mutational matrix as a (sparse) matrix
#' @param mode indicates the used input format. Must be either "CAPRI" or "CAPRIpop"
#'
#' @return a list object representing the graph computed by CIMICE with the
#' structure `list(topology = g, weights = W, labels = labels, freqs=freqs)`
#'
#' @examples
#' quick_run(example_dataset())
#'
#' @export quick_run
quick_run <- function(dataset, mode="CAPRI"){
    # preprocess
    preproc <- NULL
    if(mode == "CAPRI"){
        preproc <- dataset_preprocessing(dataset)
    }else if(mode == "CAPRIpop"){
        preproc <- dataset_preprocessing_population(dataset)
    }else{
        stop("Unsupported input mode", mode, "use CAPRI o CAPRIpop")
    }
    samples <- preproc[["samples"]]
    freqs   <- preproc[["freqs"]]
    labels  <- preproc[["labels"]]
    genes   <- preproc[["genes"]]
    matching_samples   <- preproc[["matching_samples"]]
    g <- graph_non_transitive_subset_topology(samples,labels)
    W <- compute_weights_default(g, freqs)
    list(topology = g, weights = W, labels = labels, freqs=freqs, matching_samples=matching_samples)
}

#' Simple test of the CIMICE pipeline
#'
#' Tests the whole pipeline and text (dot) output on a simple test case
#'
#' @return TRUE if the test is successful
#'
#' @examples
#' require(dplyr)
#' require(purrr)
#' test.pipeline()
#'
test.pipeline <- function(){
    require(dplyr)
    require(purrr)
    ans <- to_dot(quick_run(example_dataset()), mode = "geneIDs")
    precom_ans <- "digraph G{\n1[label=\"D\"]\n2[label=\"A\"]\n3[label=\"A, D\"]\n4[label=\"A, C, D\"]\n5[label=\"A, B, D\"]\n6[label=\"Clonal\"]\n6 -> 1[label=\"0.333\"]\n6 -> 2[label=\"0.667\"]\n1 -> 3[label=\"1\"]\n2 -> 3[label=\"1\"]\n3 -> 4[label=\"0.25\"]\n3 -> 5[label=\"0.75\"]\n}"
    test_that("full pipeline", {
        expect_equal(ans, precom_ans)
    })
}

test.pipeline()

#' Create mutational matrix from MAF file
#' 
#' Read a MAF (Mutation Annotation Format) file and create
#' a Mutational Matrix combining gene and sample IDs.
#' 
#' @param path path to MAF file
#' @param ... other maftools::mutCountMatrix arguments
#' 
#' @return the mutational (sparse) matrix associated to the MAF file
#' 
#' @examples 
#' read_MAF(system.file("extdata", "paac_jhu_2014_500.maf", package = "CIMICE", mustWork = TRUE))
#' 
#' @export read_MAF
read_MAF <- function(path, ...){
    maf <- read.maf(maf = path)
    # extract mutational matrix
    mat <- maf %>% mutCountMatrix() %>% 
        # apply(c(1,2), function(x) ifelse(x>0,1,0)) %>%
        apply(c(1,2), function(x) as.numeric(x>0)) %>%
        t 
    # make boolean
    mat <- Matrix(mat, sparse = TRUE) 
    mat
} 

#' Read dataset from an R matrix
#' 
#' also converts that matrix to a sparse matrix
#'
#' @param mat a boolean mutational matrix
#'  
#' @return the sparse mutational matrix to be used as CIMICE's input
#' 
#' @examples 
#' m <- matrix(c(0,0,1,1,0,1,1,1,1), ncol=3)
#' colnames(m) <- c("A","B","C")
#' rownames(m) <- c("S1", "S2", "S3")
#' read_matrix(m)
#' @export read_matrix
read_matrix <- function(mat){
    Matrix(mat, sparse=TRUE)
}

#' Gradually read a file from disk
#'
#' This function creates a reader to read a text file in batches (or chunks). 
#' It can be used for very large files that cannot fit in RAM.
#'
#' @param file_path path to large file
#'
#' @return a list-object containing the function `read` to 
#' read lines from the given file, and `close` to close the
#' connection to the file stream.
#' 
#' @examples 
#' # open connection to file
#' reader <- chunk_reader(
#'     system.file("extdata", "paac_jhu_2014_500.maf", package = "CIMICE", mustWork = TRUE)
#' )
#' 
#' while(TRUE){
#'     # read a chunk
#'     chunk <- reader$read(10)
#'     if(length(chunk) == 0){
#'         break
#'     }    
#'     # --- process chunk ---
#' }
#' # close connection
#' reader$close()
#' 
#' @export chunk_reader
chunk_reader <- function(file_path){
    f <- file(file_path)
    open(f)
    
    list(
        # Returns a character vector of the read lines.
        # The returned vector has length 0 if there is nothing to read.
        # This function keeps track of the current position in the file
        read = function(max_chunk = 10000){
            readLines(f, n = max_chunk)
        },
        # close connection to the file stream
        close = function(){
            close(f)
        }
    )
}

#' Add samples and genes names to a mutational matrix 
#'
#' Given M mutational matrix, add samples as row names, and genes as 
#' column names. If there are repetitions in row names, these are
#' solved by adding a sequential identifier to the names.
#' 
#' @param M mutational matrix
#' @param samples list of sample names
#' @param genes list of gene names
#'
#' @return N with the set row and column names
#' 
#' @examples 
#' require(Matrix)
#' genes <- c("A", "B", "C")
#' samples <- c("S1", "S2", "S2")
#' M <- Matrix(c(0,0,1,0,0,1,0,1,1), ncol=3, sparse=TRUE, byrow = TRUE)
#' 
#' annotate_mutational_matrix(M, samples, genes)
#' 
#' @export annotate_mutational_matrix
annotate_mutational_matrix <- function(M, samples, genes){
    assertthat::are_equal(ncol(M), length(genes))
    assertthat::are_equal(nrow(M), length(samples))
    # manage repeated rownames
    tryCatch({
        rownames(M) <- samples
    },
    error = function(e){
        # in case of repetitions, add a sequential number to the sample labels
        rownames(M) <- map2_chr(samples, seq(1,nrow(M)), ~ paste(.x, .y, sep="_"))
    })
    colnames(M) <- genes
    M
}


#' Read "CAPRI" file from a string
#'
#' Read a "CAPRI" formatted file, from a text string
#'
#' @param txt string in valid "CAPRI" format
#'
#' @return the described mutational matrix as a (sparse) matrix
#'
#' @examples
#' read_CAPRI_string("
#' s\\g A B C D
#' S1 0 0 0 1
#' S2 1 0 0 0
#' S3 1 0 0 0
#' S4 1 0 0 1
#' S5 1 1 0 1
#' S6 1 1 0 1
#' S7 1 0 1 1
#' S8 1 1 0 1
#' ")
#'
#' @export read_CAPRI_string
read_CAPRI_string <- function(txt){
    # read string as csv
    df <- read.csv(text=txt, sep="", strip.white = TRUE, blank.lines.skip = TRUE, row.names = NULL)
    # separate genes, samples and mutational matrix
    samples <- (df[,1])
    genes <- colnames(df)[-1]
    mutmatrix <- df[,-1] %>% as.matrix %>% Matrix(sparse = TRUE) 
    # glue components together
    annotate_mutational_matrix(mutmatrix, samples, genes)
}


#' Read "CAPRIpop" file from a string
#'
#' Read a "CAPRIpop" formatted file, from a text string
#'
#' @param txt string in valid "CAPRIpop" format
#'
#' @return the described mutational matrix as a (sparse) matrix
#'
#' @examples
#' read_CAPRIpop_string("
#' s\\g A B C D freqs
#' S1 0 0 0 1 0.1
#' S2 1 0 0 0 0.1
#' S3 1 0 0 0 0.2
#' S4 1 0 0 1 0.3
#' S5 1 1 0 1 0.05
#' S6 1 1 0 1 0.1
#' S7 1 0 1 1 0.05
#' S8 1 1 0 1 0.01
#' ")
#'
#' @export read_CAPRIpop_string
read_CAPRIpop_string <- function(txt){
    # read string as csv
    df <- read.csv(text=txt, sep="", strip.white = TRUE, blank.lines.skip = TRUE, row.names = NULL)
    # separate genes, samples and mutational matrix
    samples <- (df[,1])
    genes <- colnames(df)[c(-1,-ncol(df))]
    mutmatrix <- df[,c(-1,-ncol(df))] %>% as.matrix %>% Matrix(sparse = TRUE) 
    counts <- df[,ncol(df)]
    # glue components together
    list(matrix = annotate_mutational_matrix(mutmatrix, samples, genes), counts = as.numeric(counts))
}


#' Read a "CAPRI" file
#'
#' Read a "CAPRI" formatted file from the file system
#'
#' @param filepath path to file
#'
#' @return the described mutational matrix as a (sparse) matrix
#'
#' @examples
#' #          "pathToDataset/myDataset.CAPRI"
#' read_CAPRI(system.file("extdata", "example.CAPRI", package = "CIMICE", mustWork = TRUE))
#'
#' @export read_CAPRI
read_CAPRI <- function(filepath){
    reader <- chunk_reader(filepath)
    
    # read header, remove first column name
    genes <- (reader$read(1) %>% strsplit("\\s+"))[[1]][-1] 
    mutmatrix <- Matrix(, ncol = length(genes), nrow=0)
    samples <- c()
    
    while(TRUE){
        chunk <- reader$read()
        if(length(chunk) == 0){ 
            break
        }
        
        # convert to list of char array
        chunk <- map(chunk, ~ strsplit(., "\\s+")[[1]])
        
        samples <- c(samples, map_chr(chunk, ~ .[1]))
        
        new_rows <- map(chunk, ~ .[-1]) %>% unlist %>% map_int(~strtoi(.))
        mutmatrix <- rbind(mutmatrix, Matrix(new_rows, sparse = TRUE, ncol = length(genes), byrow = TRUE)) 
    }
    
    # close connection
    reader$close()
    annotate_mutational_matrix(mutmatrix, samples, genes)
}

#' Read a "CAPRI" file
#'
#' Read a "CAPRI" formatted file, as read_CAPRI
#'
#' @param filepath path to file
#'
#' @return the described mutational matrix as a (sparse) matrix
#'
#' @examples
#' read(system.file("extdata", "example.CAPRI", package = "CIMICE", mustWork = TRUE))
#'
#' @export read
read <- read_CAPRI

#' Read a "CAPRIpop" file
#'
#' Read a "CAPRIpop" formatted file from the file system
#'
#' @param filepath path to file
#'
#' @return a list containing the described mutational matrix as a (sparse) matrix
#'         and a list of the frequency of the genotypes
#'
#' @examples
#' #          "pathToDataset/myDataset.CAPRI"
#' read_CAPRI(system.file("extdata", "example.CAPRIpop", package = "CIMICE", mustWork = TRUE))
#'
#' @export read_CAPRIpop
read_CAPRIpop <- function(filepath){
    reader <- chunk_reader(filepath)
    
    # read header, remove first column name

    genes <- (reader$read(1) %>% strsplit("\\s+"))[[1]][-1]
    genes <- genes[-length(genes)]
    mutmatrix <- Matrix(, ncol = length(genes), nrow=0)
    rown <- c()
    counts <- c()
    samples <- c()
    
    while(TRUE){
        chunk <- reader$read()
        if(length(chunk) == 0){
            break
        }
        
        # convert to list of char array
        chunk <- map(chunk, ~ strsplit(., "\\s+")[[1]])
        
        samples <- c(samples, map_chr(chunk, ~ .[1]))
        # keep last column
        counts <- c(counts, map_chr(chunk, ~ .[length(genes)+2]))
        # remove last and first columns from the matrix
        new_rows <- map(chunk, ~ .[c(-1, -length(chunk[[1]]))]) %>% unlist %>% map_int(~strtoi(.))
        mutmatrix <- rbind(mutmatrix, Matrix(new_rows, sparse = TRUE, ncol = length(genes), byrow = TRUE)) 
    }
    
    # close connection
    reader$close()
    # list [mutational matrix, frequencies]

    list(matrix = annotate_mutational_matrix(mutmatrix, samples, genes), counts = as.numeric(counts))
}


#' Dataset line by line construction: initialization
#'
#' Initialize a dataset for "line by line" creation
#'
#' @param ... gene names (do not use '"', the input
#' is automatically converted to strings)
#'
#' @return a mutational matrix without samples structured as (sparse) matrix
#'
#' @examples
#' make_dataset(APC,P53,KRAS)
#'
#' @export make_dataset
make_dataset <- function(...){
    mutmatrix <- Matrix(, ncol = length(enexprs(...)), nrow=0)
    colnames(mutmatrix) <- enexprs(...)
    mutmatrix
}

#' Dataset line by line construction: add a sample
#'
#' Add a sample (a row) to an existing dataset.
#' This procedure is meant to be used with the "%>%" operator
#'
#' @param mutmatrix an existing (sparse) matrix (mutational matrix)
#' @param sampleName the row (sample) name
#' @param ... sample's genotype (0/1 numbers)
#'
#' @return the modified (sparse) matrix (mutational matrix)
#'
#' @examples
#'
#' require(dplyr)
#' make_dataset(APC,P53,KRAS)   %>%
#'     update_df("S1", 1, 0, 1) %>%
#'     update_df("S2", 1, 1, 1)
#'
#'
#' @export update_df
update_df <- function(mutmatrix, sampleName, ...){
    samples <- rownames(mutmatrix)
    mutmatrix <- rbind(mutmatrix, unlist(enexprs(...)))
    rownames(mutmatrix) <- c(samples, sampleName)
    mutmatrix
}

utils::globalVariables(c("sums"))

#' Histogram of genes' frequencies
#'
#' Create the histogram of the genes' mutational frequencies
#'
#' @param mutmatrix input dataset (mutational matrix)
#' @param binwidth binwidth parameter for the histogram (as in ggplot)
#'
#' @examples
#' gene_mutations_hist(example_dataset(), binwidth = 10)
#'
#' @return the newly created histogram
#'
#' @export gene_mutations_hist
gene_mutations_hist <- function(mutmatrix, binwidth = 1){
    df <- mutmatrix %>% as.matrix %>% as.data.frame 
    # transpose dataframe
    temp.df <- as.data.frame(t(df[,seq(1,nrow(t(df)))]))
    # compute number of alteration per gene
    temp.df$sums <- rowSums(temp.df[,seq(1,ncol(temp.df))], na.rm = TRUE)
    # count number of genes
    nGenes <- nrow(temp.df)
    # create the histogram
    ggplot(temp.df, aes(x=sums)) +
        geom_histogram(color="black", fill="white", binwidth = binwidth) +
        labs(title = glue("Number of mutations per gene ({nGenes} genes)"),
        x = "Number of mutations", y = "Count")
}

#' Histogram of samples' frequencies
#'
#' Create the histogram of the samples' mutational frequencies
#'
#' @param mutmatrix input dataset (mutational matrix)
#' @param binwidth binwidth parameter for the histogram (as in ggplot)
#'
#' @examples
#' sample_mutations_hist(example_dataset(), binwidth = 10)
#'
#' @return the newly created histogram
#'
#' @export sample_mutations_hist
sample_mutations_hist <- function(mutmatrix, binwidth = 1){
    df <- mutmatrix %>% as.matrix %>% as.data.frame
    temp.df <- df
    # compute number of alteration per sample
    temp.df$sums <- rowSums(temp.df[,seq(1,ncol(df))], na.rm = TRUE)
    # count number of samples
    nSample <- nrow(temp.df)
    # create the histogram
    ggplot(temp.df, aes(x=sums)) +
        geom_histogram(color="black", fill="white", binwidth = binwidth) +
        labs(title = glue("Number of mutations per sample ({nSample} samples)"),
            x = "Number of mutations",
            y = "Counts")
}

#' Correlation plot from mutational matrix
#'
#' Prepare correlation plot based on a mutational matrix
#'
#' @param mutmatrix input dataset
#'
#' @return the computed correlation plot
#'
#' @examples
#' corrplot_from_mutational_matrix(example_dataset())
#'
#' @export corrplot_from_mutational_matrix
corrplot_from_mutational_matrix <- function(mutmatrix){
    df <- mutmatrix %>% as.matrix %>% as.data.frame
    # prepare correlation matrix
    corr <- round(cor(df), 1)
    corr[is.nan(corr)] <- 0
    corr[is.na(corr)] <- 0
    corr[corr == Inf] <- 0
    # prepare corrplot
    ggcorrplot(corr, hc.order = TRUE)
}

#' Gene based correlation plot
#'
#' Prepare a correlation plot computed from genes' perspective
#' using a mutational matrix
#'
#' @param mutmatrix input dataset (mutational matrix)
#'
#' @return the computed correlation plot
#'
#' @examples
#' corrplot_genes(example_dataset())
#'
#' @export corrplot_genes
corrplot_genes <- corrplot_from_mutational_matrix

#' Sample based correlation plot
#'
#' Prepare a correlation plot computed from samples' perspective
#' using a mutational matrix
#'
#' @param mutmatrix input dataset (mutational matrix)
#'
#' @return the computed correlation plot
#'
#' @examples
#' corrplot_samples(example_dataset())
#'
#' @export corrplot_samples
corrplot_samples <- function (mutmatrix)
    corrplot_from_mutational_matrix(t(mutmatrix))

utils::globalVariables(c("from", "label", "matching_sample", "rows", "to", "tot", "weight", "weight.x", "weight.y"))

#' Create a stub for a generator 
#'
#' Create a generator topology directly from a dataset.
#' The topology will follow the subset relation.
#' 
#' @param dataset A compacted CIMICE dataset
#'
#' @return a generator, with weight = 0 for all the edges
#' 
#' @examples 
#' make_generator_stub(example_dataset())
#' 
#' @export make_generator_stub
make_generator_stub <- function(dataset){
  graph_data <- example_dataset() %>% dataset_preprocessing()
  generator <- graph_non_transitive_subset_topology(graph_data$samples, graph_data$labels) %>% 
    as_tbl_graph() %>% 
    mutate(matching_sample = graph_data$matching_samples)
  for(label in generator %>% pull(label) ){
    generator <- generator %>% bind_edges(from = label, to = label, node_key = "label")
  }
  
  generator <- generator %E>% mutate(weight = 0)
  generator <- activate(generator, what="nodes")
  list(graph = generator, data = graph_data)
}

#' Add edge weights to a generator
#' 
#' @param generator a generator 
#' @param f_t_w_list a list of triplets (from, to, list), the triplets must not 
#' be nested in the list. For example list("A","B",0.3, "B", "C", 0.2) is a valid
#' input.
#' @param by "labels" or "samples" to use gene labels or sample labels as references for
#' edge identifiers.
#' 
#' 
#' @return the generator with the modified edges (invalid edges are ignored)
#'   
#' @examples 
#' require(dplyr)
#' 
#' example_dataset() %>%
#'  make_generator_stub() %>% 
#'  set_generator_edges(
#'    list(
#'     "D", "A, D", 1 , 
#'     "A", "A, D", 1 , 
#'     "A, D", "A, C, D", 1 , 
#'     "A, D", "A, B, D", 1 , 
#'     "Clonal", "D", 1 , 
#'     "Clonal", "A", 1 , 
#'     "D", "D", 1 , 
#'     "A", "A", 1 , 
#'     "A, D", "A, D", 1 , 
#'     "A, C, D", "A, C, D", 1 , 
#'     "A, B, D", "A, B, D", 1 , 
#'     "Clonal", "Clonal", 1 
#'   ))
#'    
#' @export set_generator_edges      
set_generator_edges <- function(generator, f_t_w_list, by = "labels"){
  
  # create columns for the data frame
  type <- 1
  from_l <- list()
  to_l <- list()
  weight_l <- c()
  for(el in f_t_w_list){
    if(type == 1){
      from_l <- c(from_l, el)
      type <- 2
    }else if(type == 2){
      to_l <- c(to_l, el)
      type <- 3
    }else{
      assert_that(is.numeric(el), msg="weights must be numeric")
      weight_l <- c(weight_l, el)
      type <- 1
    }
  }
  assert_that(length(from_l) == length(to_l) & length(to_l) == length(weight_l), 
              msg="The sequence must be multiple of three (from, to, weight)")
  
  # get node ids by label/sample_label
  if(by == "samples"){
    id_col <- "matching_samples"
  }else{
    id_col <- "label"
  }
  
  from_l <- map_int(from_l, ~ ifelse(is.numeric(.),
                                     ., 
                                     which((generator$graph %N>% pull(!!id_col)) == .)))
  to_l <- map_int(to_l, ~ ifelse(is.numeric(.),
                                     ., 
                                     which((generator$graph %N>% pull(!!id_col)) == .)))
  
  # create the weight dataframe
  weights_df <- data.frame(from=from_l,to=to_l,weight=weight_l)
  
  # add weights
  generator$graph <- generator$graph %E>% left_join(weights_df, by=c("from","to")) %>% 
    mutate(weight = ifelse(is.na(weight.y), weight.x, weight.y)) %>% 
    select(-weight.x, -weight.y)
  
  generator$graph <- activate(generator$graph, what="nodes")
  generator
}

#' Prepare a command to add edge weights to a generator
#' 
#' Prints a string in the form of the command that 
#' sets weights for all the edges of this generator.
#'
#' @param generator a generator 
#' @param by "labels" or "samples" to use gene labels or sample labels as references for
#' edge identifiers.
#' 
#' @return NULL (the string with the function calls is printed on the stdout) 
#'
#' @examples 
#' require(dplyr)
#' example_dataset() %>% 
#'   make_generator_stub() %>%
#'   prepare_generator_edge_set_command()
#'
#' @export prepare_generator_edge_set_command
prepare_generator_edge_set_command <- function(generator, by="labels"){
  edges <- generator$graph %E>% data.frame()
  generator$graph %N>% print
  if(by == "samples"){
    names_vec <- generator$graph %N>% pull(matching_sample)
  }else{
    names_vec <- generator$graph %N>% pull(label)
  }
  out <- c("set_generator_edges(\n  list(\n")
  for(row in seq(1,nrow(edges))){
    out <- c(out, "    ")
    out <- c(out, paste(
      paste('"', names_vec[edges[row, "from"]], '"', sep=""),
      paste('"', names_vec[edges[row, "to"]], '"', sep=""),
      "_", sep=", "))
    if(row < nrow(edges)){
      out <- c(out, ",")
    }
    out <- c(out, "\n")
  }
  out <- c(out, "))\n")
  paste(out) %>% cat
}

#' Plot a generator
#'
#' Simple ggraph interface to draw a generator 
#' 
#' @param generator a generator
#' 
#' @return a basic plot of this generator
#'
#' @examples 
#' require(dplyr)
#' 
#' example_dataset() %>%
#'  make_generator_stub() %>% 
#'  set_generator_edges(
#'    list(
#'     "D", "A, D", 1 , 
#'     "A", "A, D", 1 , 
#'     "A, D", "A, C, D", 1 , 
#'     "A, D", "A, B, D", 1 , 
#'     "Clonal", "D", 1 , 
#'     "Clonal", "A", 1 , 
#'     "D", "D", 1 , 
#'     "A", "A", 1 , 
#'     "A, D", "A, D", 1 , 
#'     "A, C, D", "A, C, D", 1 , 
#'     "A, B, D", "A, B, D", 1 , 
#'     "Clonal", "Clonal", 1 
#'   )) %>% 
#'   finalize_generator %>% 
#'   plot_generator
#'  
#' @export plot_generator
plot_generator <- function(generator){
  ggraph(generator$graph, layout = "sugiyama") +
    geom_node_point() +
    geom_node_label(aes(label=label)) +
    geom_edge_link(
      aes(label=weight, start_cap = label_rect(node1.label),
          end_cap = label_rect(node2.label)),
      arrow = arrow(length = unit(4, 'mm')),
      label_dodge = unit(2.5, 'mm'),
      angle_calc = 'along'
    ) +
    geom_edge_loop(
      aes(label=weight, start_cap = label_rect(node1.label),
          end_cap = label_rect(node2.label)),
      arrow = arrow(length = unit(4, 'mm')),
      label_dodge = unit(2.5, 'mm'),
      angle_calc = 'along'
    ) 
}

#' Finalize generator normalizing edge weights
#' 
#' Checks if a generator can be normalized so that it 
#' actually is a Markov Chain 
#' 
#' @param generator a generator 
#' 
#' @return A generator with edge weights that respect DTMC definition
#' 
#' @examples 
#' require(dplyr)
#' 
#' example_dataset() %>%
#'  make_generator_stub() %>% 
#'  set_generator_edges(
#'    list(
#'     "D", "A, D", 1 , 
#'     "A", "A, D", 1 , 
#'     "A, D", "A, C, D", 1 , 
#'     "A, D", "A, B, D", 1 , 
#'     "Clonal", "D", 1 , 
#'     "Clonal", "A", 1 , 
#'     "D", "D", 1 , 
#'     "A", "A", 1 , 
#'     "A, D", "A, D", 1 , 
#'     "A, C, D", "A, C, D", 1 , 
#'     "A, B, D", "A, B, D", 1 , 
#'     "Clonal", "Clonal", 1 
#'   )) %>% 
#'   finalize_generator
#' 
#' @export finalize_generator
finalize_generator <- function(generator){
  normalization_df <- generator$graph %E>% 
    data.frame() %>% 
    group_by(from) %>% 
    summarise(tot = sum(weight)) 
  
  if(length(which(normalization_df %>% pull(tot) == 0)) != 0){
    stop("Some node has no exiting edge with probability greater than 0.\nCheck your call to set_generator_edges")
  }
  
  generator$graph <- generator$graph %E>% 
    left_join(normalization_df, by="from") %>% 
    mutate(weight = weight/tot) %>% 
    select(-tot)
  
  generator
}

#' Create datasets from generators
#' 
#' Simulate the DTMC associated to the generator to create a dataset
#' that reflects the genotypes of `times` cells, sampled after
#' `time_ticks` passages.
#' 
#' @param generator a generator 
#' @param time_ticks number of steps (updates) of the DTMC associated to the generato
#' @param times number of sumlated cells
#' @param starting_label node from which to start the simulation
#' @param by "labels" or "samples" to use gene labels or sample labels as references to identify the `starting_label`'s node
#' @param mode "full" to generate a matrix with `times` genotypes, "compacted" to *efficiently* create an already compacted dataset 
#' (a dataset showing the genotypes and their respective frequencies)
#'
#' @return the simulated dataset
#'
#' @examples 
#' require(dplyr)
#' 
#' example_dataset() %>%
#'   make_generator_stub() %>% 
#'   set_generator_edges(
#'     list(
#'       "D", "A, D", 1 , 
#'       "A", "A, D", 1 , 
#'       "A, D", "A, C, D", 1 , 
#'       "A, D", "A, B, D", 1 , 
#'       "Clonal", "D", 1 , 
#'       "Clonal", "A", 1 , 
#'       "D", "D", 1 , 
#'       "A", "A", 1 , 
#'       "A, D", "A, D", 1 , 
#'       "A, C, D", "A, C, D", 1 , 
#'       "A, B, D", "A, B, D", 1 , 
#'       "Clonal", "Clonal", 1 
#'   )) %>% 
#'   finalize_generator %>% 
#'   simulate_generator(3, 10)
#'
#' @export simulate_generator
simulate_generator <- function(generator, time_ticks, times, starting_label = "Clonal", by = "labels", mode="full"){
  m <- sparseMatrix(i = generator$graph %E>% pull(from),
               j = generator$graph %E>% pull(to),
               x = generator$graph %E>% pull(weight)) %>% 
    as.matrix()
  # quickly compute n_step matrix (O(log2(times))) matrix multiplications
  m <- m %^% time_ticks
  if(by == "samples"){
    names_vec <- generator$graph %N>% pull(matching_sample)
  }else{
    names_vec <- generator$graph %N>% pull(label)
  }
  v <- m[which(names_vec == starting_label),]
  
  if(mode == "full"){
    # this option easily allows to introduce FP/FN rates and missing data
    dataset <- sample(seq(1,length(v)), times, prob = v, replace = TRUE)
    generator$data$samples[dataset,]
  }else if(mode == "compacted"){
    # this option efficiently computes very large datasets
    dataset <- sample(seq(1,length(v)), times, prob = v, replace = TRUE)
    selected_genotypes <- dataset %>% unique()
    count_df <- data.frame(rows = dataset) %>% count(rows)
    sorted_count_df <- inner_join(data.frame(rows = selected_genotypes), count_df, by="rows") 
    out_matrix <- generator$data$samples[selected_genotypes,]
    list(matrix = out_matrix, 
         counts = sorted_count_df %>% pull(n),
         row_names = rownames(out_matrix))
  }else{
    stop("Invalid mode ", mode, " for simulate_generator")
  }
  
}

#' Perturbate a boolean matrix
#'
#' Given a boolean matrix, randomly 
#' add False Positives (FP), False Negatives (FN) and Missing data 
#' following user defined rates. In the final matrix, missing 
#' data is represented by the value 3.
#' 
#' Note that CIMICE does not support dataset with missing data natively, 
#' so using MIS_rate != 0 will then require some pre-processing.
#'
#' @param dataset a matrix/sparse matrix
#' @param FP_rate False Positive rate
#' @param FN_rate False Negative rate
#' @param MIS_rate Missing Data rate
#' 
#' @return the new, perturbed, matrix
#' 
#' @examples 
#' require(dplyr)
#' 
#' example_dataset() %>%
#'   make_generator_stub() %>% 
#'   set_generator_edges(
#'     list(
#'       "D", "A, D", 1 , 
#'       "A", "A, D", 1 , 
#'       "A, D", "A, C, D", 1 , 
#'       "A, D", "A, B, D", 1 , 
#'       "Clonal", "D", 1 , 
#'       "Clonal", "A", 1 , 
#'       "D", "D", 1 , 
#'       "A", "A", 1 , 
#'       "A, D", "A, D", 1 , 
#'       "A, C, D", "A, C, D", 1 , 
#'       "A, B, D", "A, B, D", 1 , 
#'       "Clonal", "Clonal", 1 
#'   )) %>% 
#'   finalize_generator %>% 
#'   simulate_generator(3, 10) %>% 
#'   perturb_dataset(FP_rate = 0.01, FN_rate = 0.1, MIS_rate = 0.12)
#'
#' @export perturb_dataset
perturb_dataset <- function(dataset, FP_rate = 0, FN_rate = 0, MIS_rate = 0){
  apply(dataset, c(1,2), function(x){
    if(rbernoulli(1, MIS_rate)){
      3
    }else if(x == 0){
      if(rbernoulli(1, FP_rate)){
        1
      }else{
        0
      }
    }else if(x == 1){
      if(rbernoulli(1, FN_rate)){
        0
      }else{
        1
      }
    }
  })
}

utils::globalVariables(c("node1.label", "node2.label"))
#' ggplot graph output
#'
#' Draws the output graph using ggplot
#'
#' @param out the output object of CIMICE (es, from quick run)
#' @param digits precision for edges' weights
#' @param ... other arguments for format_labels
#'
#' @return ggraph object representing g as described
#'
#' @examples
#' draw_ggraph(quick_run(example_dataset()))
#' 
#' @export draw_ggraph
draw_ggraph <- function(out, digits = 4, ...){
    g <- out$topology 
    W <- out$weights
    labels <- make_labels(out, ...)
    
    V(g)$label <- labels

    weights <- signif(W, digits = 4)

    ggraph(g, layout = "sugiyama") +
        geom_node_point() +
        geom_node_label(aes(label=labels)) +
        geom_edge_link(
            aes(label=weights, start_cap = label_rect(node1.label),
                end_cap = label_rect(node2.label)),
            arrow = arrow(length = unit(4, 'mm')),
            label_dodge = unit(2.5, 'mm'),
            angle_calc = 'along'
        )
}

#' Format labels for output object
#'
#' Prepare labels based on multiple identifiers so that they do not excede
#' a certain size (if they do, a simple number is used)
#' 
#' @param labels a charachter vector of the labels to manage
#' @param max_col maximum number of identifiers in a single row for a label
#' @param max_row maximum number of rows of identifiers in a label
#' 
#' @return the updated labels
#' 
#' @examples  
#' format_labels(c("A, B", "C, D, E"))
#'
#' @export format_labels
format_labels <- function(labels, max_col = 3, max_row = 3){
    # prepare basic labels as replacement of too long labels
    basic_labels <- map_chr(seq(1,length(labels)), ~ as.character(.))
    for(i in seq(1,length(labels))){
        # divide the label in tokens
        lsplit <- strsplit(labels[i], ", ")[[1]]
        if(length(lsplit) > max_col*max_row){
            # label too long
            labels[i] <- basic_labels[i]
        }else{
            # creation of the label's grid of IDs
            new_label <- ""
            col <- 1
            first_run <- TRUE
            for(l in lsplit){
                
                if(col == 1){
                    if(col == max_col){
                        col <- 1
                    }else{
                        col <- col + 1    
                    }
                    if(first_run){
                        new_label <- c(new_label, l)
                        first_run <- FALSE
                    }else{
                        new_label <- c(new_label, "\n", l)
                    }
                }else if(col == max_col){
                    col <- 1
                    new_label <- c(new_label, ", ", l)
                }else{
                    col <- col + 1
                    new_label <- c(new_label,", ", l)
                }
            }
            labels[i] <- paste(new_label, collapse = "")
        }
    }
    labels
}

#' Helper function to create labels 
#' 
#' This function helps creating labels for nodes with different information
#' 
#' @param out the output object of CIMICE (es, from quick run)
#' @param mode which labels to print: samplesIDs (matching samples), sequentialIDs (just a sequential numer), geneIDs (genotype)
#' @param max_col identifiers are represented in a max_col times max_row grid (if the number of IDs exceeds, a the sequentialID number is used instead) 
#' @param max_row identifiers are represented in a max_col times max_row grid (if the number of IDs exceeds, a the sequentialID number is used instead)
#' 
#' @return the requested labels
#'
#' @examples  
#' make_labels(quick_run(example_dataset()))
#' 
#' @export make_labels
make_labels <- function(out, mode = "samplesIDs", max_col = 3, max_row = 3){
    if(mode == "samplesIDs"){
        labels <- out$matching_samples %>% as.character %>% format_labels(max_col, max_row)
    }else if(mode == "sequentialIDs"){
        labels <- map_chr(seq(1,length(out$labels)), ~ as.character(.)) 
    }else if(mode == "geneIDs"){
        labels <- out$labels %>% as.character %>% format_labels(max_col, max_row)
    }else{
        stop("Invalid output mode:", mode)
    }
    labels
}


#' NetworkD3 graph output
#'
#' Draws the output graph using networkD3
#'
#' @param out the output object of CIMICE (es, from quick run)
#' @param ... other arguments for format_labels
#'
#' @return networkD3 object representing g as described
#'
#' @examples
#' draw_networkD3(quick_run(example_dataset()))
#'
#' @export draw_networkD3
draw_networkD3 <- function(out, ...){
    g <- out$topology 
    W <- out$weights
    labels <- make_labels(out, ...)
    
    V(g)$label <- labels
    E(g)$label <- W

    gr <- igraph_to_networkD3(g)
    gr$nodes$label <- unlist(labels, use.names=FALSE)
    # Create force directed network plot
    p<-forceNetwork(Links = gr$links, Nodes = gr$nodes,
                    Source = 'source', Target = 'target', Value = "value",
                    NodeID = 'label', Group = 'name',
                    zoom=TRUE, arrows = TRUE, opacityNoHover = FALSE)
    p
}

# #' === DEPRECATED ===
# 
# VisNetwork graph output (default)
#
# Draws the output graph using VisNetwork
#
# @param g graph to be drawn
# @param W weights on edges
# @param labels node labels
# @param original_dataset if not NULL, recover matching samples from the rownames of the original dataset
#
# @return visNetwork object representing g as described
#
# @examples
# require(dplyr)
# preproc <- example_dataset() %>% dataset_preprocessing
# samples <- preproc[["samples"]]
# freqs   <- preproc[["freqs"]]
# labels  <- preproc[["labels"]]
# genes   <- preproc[["genes"]]
# g <- graph_non_transitive_subset_topology(samples, labels)
# W <- compute_weights_default(g, freqs)
# draw_visNetwork(g,W,labels)
#
# @export draw_visNetwork
# draw_visNetwork <- function(out, original_dataset = NULL){
# 
#     V(g)$label <- labels
#     E(g)$label <- W
# 
#     gr <- igraph_to_networkD3(g)
#     gr$nodes$label <- unlist(labels, use.names=FALSE)
# 
#     labs <- seq(1,length(gr$nodes$label)) %>% map_chr(~as.character(.))
#     if(!is.null(original_dataset)){
#         matching_samples <- get_samples_lables_matches(gr, original_dataset)
#         for(i in seq(1,length(gr$nodes$label))){
#             if(length(strsplit(matching_samples[i], ",")[[1]]) <= 4){
#                 labs[i] <- matching_samples[i]
#             }
#         }
#     }
#     
#     if(is.null(original_dataset)){
#         matching_samples <- ""
#     }else{
#         matching_samples <- paste0("<br></p></font><p>Matching:<br>", matching_samples)
#     }
#    
#     nodes <- data.frame(
#         id = gr$nodes$name,
#         label = labs, #gr$nodes$label,
#         shadow = TRUE,
#         shape = "box",
#         title = paste0("<font color=\"black\"><p>Node:<br>",
#                     gr$nodes$label,
#                     matching_samples)
#     )
#     # compute genotype difference from source to destination
#     diff <- map2(gr$links$source, gr$links$target,
#                 function(x, y){
#                     a <- unlist(strsplit(gr$nodes$label[x+1], ", "))
#                     b <- unlist(strsplit(gr$nodes$label[y+1], ", "))
#                     d <- setdiff(b,a)
#                     d
#                 })
# 
#     edges <- data.frame(
#         from = gr$links$source+1,
#         to = gr$links$target+1,
#         arrows = "to",
#         shadow = TRUE,
#         label = signif(gr$links$value, digits = 4),
#         title = paste0("<font color=\"black\"><p>Edge:<br> from: ",
#                     gr$nodes$label[gr$links$source+1] ,"<br>to : ",
#                     gr$nodes$label[gr$links$target+1] ,
#                     "<br>New mutations:<br>",
#                     diff, "</p></font>")
#     )
# 
#     grp <- toVisNetworkData(g, idToLabel = TRUE)
# 
#     visNetwork(nodes, edges)
# }

#' VisNetwork graph output (default)
#'
#' Draws the output graph using VisNetwork
#'
#' @param out the output object of CIMICE (es, from quick run)
#' @param ... other arguments for format_labels
#'
#' @return visNetwork object representing g as described
#'
#' @examples
#' draw_visNetwork(quick_run(example_dataset()))
#'
#' @export draw_visNetwork
draw_visNetwork <- function(out, ...){
    g <- out$topology 
    W <- out$weights
    labels <- make_labels(out, ...)
    
    V(g)$label <- out$labels
    E(g)$label <- W
    
    gr <- igraph_to_networkD3(g)
    gr$nodes$label <- unlist(out$labels, use.names=FALSE)
    
    nodes <- data.frame(
        id = gr$nodes$name,
        label = labels, #gr$nodes$label,
        shadow = TRUE,
        shape = "box",
        title = paste0("<font color=\"black\"><p>Genotype:<br>",
                       gr$nodes$label, 
                       "<br><p>Matching samples:<br>",
                       out$matching_samples)
    )
    # compute genotype difference from source to destination
    diff <- map2(gr$links$source, gr$links$target,
                 function(x, y){
                     a <- unlist(strsplit(gr$nodes$label[x+1], ", "))
                     b <- unlist(strsplit(gr$nodes$label[y+1], ", "))
                     d <- setdiff(b,a)
                     d
                 })
    
    edges <- data.frame(
        from = gr$links$source+1,
        to = gr$links$target+1,
        arrows = "to",
        shadow = TRUE,
        label = signif(gr$links$value, digits = 4),
        title = paste0("<font color=\"black\"><p>Edge:<br> from:",
                       gr$nodes$label[gr$links$source+1] ,"<br>to : ",
                       gr$nodes$label[gr$links$target+1] ,
                       "<br>New mutations:<br>",
                       diff, "</p></font>")
    )
    
    grp <- toVisNetworkData(g, idToLabel = TRUE)
    
    visNetwork(nodes, edges) # %>% visHierarchicalLayout(sortMethod="directed")
}


# === DEPRECATED ===
#  
# Get associations between graph nodes and the samples of the original dataset
# 
# @param gr a networkD3 graph
# @param original_dataset a dataset with row and column names (samples, genes)
# 
# @return a character vector of comma separated sample identifiers. 
# These lists consists of the identifiers of the samples that match a certain node in the graph.
# (follows node's orderd in the graph).
#  
# @examples
# require(dplyr)
# require(networkD3)
# out <- quick_run(example_dataset())
# gr <- igraph_to_networkD3(out$topology)
# gr$nodes$label <- unlist(out$labels, use.names=FALSE) 
# get_samples_lables_matches(gr, example_dataset())
#
# @export get_samples_lables_matches
# get_samples_lables_matches <- function(gr, original_dataset){
#     # prepare a matrix that will report for each pair (graph_node, sample) if the sample belongs to that graph_node 
#     match_matrix <- matrix(rep(0, times=length(gr$nodes$name)*nrow(original_dataset)), ncol = nrow(original_dataset))
#     # extract the list of considered genes in this analysis
#     considered_genes <- strsplit( paste(gr$nodes$label, collapse = ", "), ",\\s*")[[1]] %>% unique 
#     considered_genes <- considered_genes[considered_genes!="Clonal"]
#     # find pairs (node, sample) with complete matching on wt/mutated genes
#     for(node_i in seq(1,length(gr$nodes$name))){
#         for(sample in seq(1,nrow(original_dataset))){
#             matching <- TRUE
#             present_genes <- strsplit(gr$nodes$label[[node_i]] , ",\\s*")[[1]]
#             for(gene in considered_genes){
#                 if(original_dataset[sample,gene] != gene %in% present_genes){
#                     matching <- FALSE
#                 }
#             }
#             match_matrix[node_i, sample] <- matching
#         }
#     }
#     # prepare strings
#     matching_samples <- rep("", times = length(gr$nodes$name))
#     for(node_i in seq(1,length(gr$nodes$name))){
#         matching_samples[node_i] <- rownames(original_dataset)[which(match_matrix[node_i, ] == TRUE)] %>% paste(collapse = ", ")
#     }
#     matching_samples
# }


#' Dot graph output
#'
#' Represents this graph in dot format (a textual output format)
#'
#' @param out the output object of CIMICE (es, from quick run)
#' @param ... other arguments for format_labels
#'
#' @return a string representing the graph in dot format
#'
#' @examples
#' to_dot(quick_run(example_dataset()))
#'
#' @export to_dot
to_dot <- function(out, ...){
    g <- out$topology 
    W <- out$weights
    labels <- make_labels(out, ...)
    
    V(g)$label <- labels
    E(g)$label <- W

    str <- paste(
        "digraph G{",
        paste(
            imap_chr(V(g)$label, ~ paste( .y, '[label="', .x ,'"]', sep="") ),
            collapse = "\n"),
        paste(
            imap_chr(E(g), ~ paste(tail_of(g, E(g)[[.y]]), " -> ", head_of(g, E(g)[[.y]]), '[label="', round(W[.y], digits = 3) , '"]', sep="" )),
            collapse = "\n"),
        "}",
        sep="\n"
    )

    str
}

# PD global structures
weight.env <- new.env()
weight.env$upWeights <- NULL
weight.env$downWeights <- NULL

#' Up weights computation
#'
#' Computes the up weights formula using
#' a Dinamic Programming approach (starting call),
#' see vignettes for further explaination.
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#'
#' @return a vector containing the Up weights for each edge
#'
#' @examples
#' require(dplyr)
#' require(igraph)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' # prepare adj matrix
#' A <- as.matrix(as_adj(g))
#' # pre-compute exiting edges from each node
#' no.of.children <- get_no_of_children(A,g)
#' computeUPW(g, freqs, no.of.children, A)
#'
#' @export computeUPW
computeUPW <- function(g, freqs, no.of.children, A){
    # create data structure for Dynamic Programming
    weight.env$upWeights <- seq(-1, -1, length.out = length(E(g)))
    weight.env$upWeights <- map_dbl( seq(1,length(E(g))), function(x)
        computeUPW_aux(g, x, freqs, no.of.children, A) )
    weight.env$upWeights
}

#' Up weights computation (aux)
#'
#' Computes the up weights formula using
#' a Dinamic Programming approach (recursion),
#' see vignettes for further explaination.
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param edge the currently considered edge
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#'
#' @return a vector containing the Up weights for each edge
#'
computeUPW_aux = function(g, edge, freqs, no.of.children, A) {
    # check if this recursion was already computed
    if (weight.env$upWeights[edge] != -1) return(weight.env$upWeights[edge])
    # get source and destination of the currently considered edge
    source = as_ids(tail_of(g,edge))
    destination = as_ids(head_of(g,edge))
    # extract number of children of the source
    D = no.of.children[source]
    # observed genotype frequency of the source
    P = freqs[source]

    # compute weight
    W <- 0
    # recursion on predecessor
    if(length(which(A[,source] >= 1)) != 0){
        # consider edges entering in the source node
        edges.entering.in.source <-
            lapply( as.list(which(A[,source] >= 1)),
                    function (x) c(x,source) )
        # get edge IDs
        edges.entering.in.source <-
            get.edge.ids(g, unlist(edges.entering.in.source))
        # recurr
        W <- sum(map_dbl(edges.entering.in.source ,
                        function (x)
                            computeUPW_aux(g, x, freqs, no.of.children, A)))
    }
    # compute formula
    weight.env$upWeights[edge] <- (1/D) * (P+W)

    return(weight.env$upWeights[edge])
}

#' Up weights normalization
#'
#' Normalizes up weights so that the sum
#' of weights of edges entering in a node is 1
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#' @param upWeights Up weights as computed by computeUPW
#'
#' @return a vector containing the normalized Up weights for each edge
#'
#' @examples
#' require(dplyr)
#' require(igraph)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' # prepare adj matrix
#' A <- as.matrix(as_adj(g))
#' # pre-compute exiting edges from each node
#' no.of.children <- get_no_of_children(A,g)
#' upWeights <- computeUPW(g, freqs, no.of.children, A)
#' normalizeUPW(g, freqs, no.of.children, A, upWeights)
#'
#' @export normalizeUPW
normalizeUPW <- function(g, freqs, no.of.children, A, upWeights) {
    normUpWeights <- seq(-1, -1, length.out = length(E(g)))

    for ( v in V(g) ){
        # almeno un arco entrante
        if(length(which(A[,v] >= 1)) != 0){
            edges.entering.in.source <-
                lapply( as.list(which(A[,v] >= 1)), function (x) c(x,v) )
            edges.entering.in.source <-
                get.edge.ids(g, unlist(edges.entering.in.source))
            normVal <-
                sum( map_dbl( edges.entering.in.source ,
                            function (x) upWeights[x]))
            for(e in edges.entering.in.source){
                if(normVal == 0){ # è l'unico arco
                    normUpWeights[e] <- 1
                }else{
                    normUpWeights[e] <- upWeights[e]/normVal
                }
            }
        }
    }
    normUpWeights
}

#' Down weights computation
#'
#' Computes the Down weights formula using
#' a Dinamic Programming approach (starting call),
#' see vignettes for further explaination.
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#' @param normUpWeights normalized up weights as computed by normalizeUPW
#'
#' @return a vector containing the Up weights for each edge
#'
#' @examples
#' require(dplyr)
#' require(igraph)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' # prepare adj matrix
#' A <- as.matrix(as_adj(g))
#' # pre-compute exiting edges from each node
#' no.of.children <- get_no_of_children(A,g)
#' upWeights <- computeUPW(g, freqs, no.of.children, A)
#' normUpWeights <- normalizeUPW(g, freqs, no.of.children, A, upWeights)
#' computeDWNW(g, freqs, no.of.children, A, normUpWeights)
#'
#' @export computeDWNW
computeDWNW <- function(g, freqs, no.of.children, A, normUpWeights){
    # create Dynamic Programming data structure
    weight.env$downWeights <- seq(-1, -1, length.out = length(E(g)))
    weight.env$downWeights <- map_dbl( seq(1,length(E(g))) , function(x)
        computeDWNW_aux(g, x, freqs, no.of.children, A, normUpWeights) )
    weight.env$downWeights
}

#' Down weights computation (aux)
#'
#' Computes the Down weights formula using
#' a Dinamic Programming approach (recursion),
#' see vignettes for further explaination.
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param edge the currently considered edge
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#' @param normUpWeights normalized up weights as computed by normalizeUPW
#'
#' @return a vector containing the Up weights for each edge
#'
computeDWNW_aux = function(g, edge, freqs, no.of.children, A, normUpWeights) {
    # check if this recursion was already computed
    if (weight.env$downWeights[edge] > -1) return(weight.env$downWeights[edge])
    # get source and destination of the currently considered edge
    source = as_ids(tail_of(g,edge))
    destination = as_ids(head_of(g,edge))

    UPW = normUpWeights[edge]
    P = freqs[destination]

    W <- 0
    # consider only nodes with exiting edges
    if(length(which(A[destination,] >= 1)) != 0){
        edges.exiting.destination <-
            lapply( as.list(which(A[destination,] >= 1)),
                    function (x) c(destination,x) )
        edges.exiting.destination <-
            get.edge.ids(g, unlist(edges.exiting.destination))
        W <- sum(map_dbl(
            edges.exiting.destination ,
            function (x)
                computeDWNW_aux(g, x, freqs, no.of.children, A, normUpWeights)))
    }
    # compute formula
    weight.env$downWeights[edge] <- (UPW) * (P+W)
    return(weight.env$downWeights[edge])
}

#' Down weights normalization
#'
#' Normalizes Down weights so that the sum
#' of weights of edges exiting a node is 1
#'
#' @param g graph (a Directed Acyclic Graph)
#' @param freqs observed genotype frequencies
#' @param no.of.children number of children for each node
#' @param A adjacency matrix of G
#' @param downWeights Down weights as computed by computeDWNW
#'
#' @return a vector containing the normalized Down weights for each edge
#'
#' @examples
#' require(dplyr)
#' require(igraph)
#' preproc <- example_dataset() %>% dataset_preprocessing
#' samples <- preproc[["samples"]]
#' freqs   <- preproc[["freqs"]]
#' labels  <- preproc[["labels"]]
#' genes   <- preproc[["genes"]]
#' g <- graph_non_transitive_subset_topology(samples, labels)
#' # prepare adj matrix
#' A <- as.matrix(as_adj(g))
#' # pre-compute exiting edges from each node
#' no.of.children <- get_no_of_children(A,g)
#' upWeights <- computeUPW(g, freqs, no.of.children, A)
#' normUpWeights <- normalizeUPW(g, freqs, no.of.children, A, upWeights)
#' downWeights <- computeDWNW(g, freqs, no.of.children, A, normUpWeights)
#' normalizeUPW(g, freqs, no.of.children, A, downWeights)
#'
#' @export normalizeDWNW
normalizeDWNW <- function(g, freqs, no.of.children, A, downWeights){
    normDownWeights <- seq(-1, -1, length.out = length(E(g)))

    for ( v in V(g) ){
        if(length(which(A[v,] >= 1)) != 0){
            edges.exiting.source <-
                lapply( as.list(which(A[v,] >= 1)), function (x) c(v,x))
            edges.exiting.source <-
                get.edge.ids(g, unlist(edges.exiting.source))
            normVal <-
                sum(map_dbl(edges.exiting.source,
                            function(x) downWeights[x]))
            for(e in edges.exiting.source){
                if(normVal == 0){
                    # if this is the only edge
                    normDownWeights[e] <- 1
                }else{
                    normDownWeights[e] <- downWeights[e]/normVal
                }
            }
        }
    }
    normDownWeights
}

#' CIMICE Package
#'
#' CIMICE-R: (Markov) Chain Method to Infer Cancer Evolution
#'
#' @description     R implementation of the CIMICE tool.
#' CIMICE is a tool in the field of tumor phylogenetics and
#' its goal is to build a Markov Chain (called Cancer Progression Markov Chain, CPMC) in order to model tumor subtypes evolution.
#' The input of CIMICE is a Mutational Matrix, so a boolean matrix representing altered genes in a
#' collection of samples. These samples are assumed to be obtained with single-cell DNA analysis techniques and
#' the tool is specifically written to use the peculiarities of this data for the CMPC construction.
#' See `https://github.com/redsnic/tumorEvolutionWithMarkovChains/tree/master/GenotypeEvolutionPaths` for the
#' original Java version of this tool.
#'
#' @author Nicolò Rossi \email{olocin.issor@gmail.com}
#'
#' @docType package
#' @name CIMICE
#'
#' @importFrom dplyr select enexprs select_if %>% ungroup summarise group_by_all n mutate_all count group_by 
#' @importFrom dplyr inner_join left_join mutate pull
#' @importFrom ggplot2 ggplot geom_histogram aes arrow unit labs
#' @importFrom glue glue
#' @importFrom tidyr drop_na
#' @importFrom igraph V E as_ids tail_of head_of get.edge.ids
#' @importFrom igraph graph_from_adjacency_matrix as_edgelist
#' @importFrom igraph graph_from_edgelist as_adj E<- V<-
#' @importFrom networkD3 igraph_to_networkD3 forceNetwork
#' @importFrom visNetwork toVisNetworkData visNetwork visHierarchicalLayout
#' @importFrom ggcorrplot ggcorrplot
#' @importFrom purrr map2 map_dbl imap_chr map map_chr map_int map2_chr rbernoulli
#' @importFrom ggraph ggraph geom_node_point geom_node_label geom_edge_loop
#' @importFrom ggraph geom_edge_link label_rect
#' @importFrom stats cor
#' @importFrom utils read.csv
#' @importFrom maftools read.maf mutCountMatrix
#' @importFrom assertthat are_equal assert_that
#' @importFrom tidygraph %E>% %N>% activate as_tbl_graph bind_edges
#' @importFrom expm %^%
#' @import Matrix
#'
NULL

---
title: "CIMICE-R: (Markov) Chain Method to Infer Cancer Evolution"
author: 
    -   name: "Nicolò Rossi"
        affiliation: "Lab. of Computational Biology and Bioinformatics, Department of Mathematics, Computer Science and Physics, University of Udine"
        email: "olocin.issor@gmail.com"
date: "6/04/2021"
geometry: margin=2cm
output:
    BiocStyle::html_document:
        toc_float: true
        df_print: kable
vignette: >
    %\VignetteEncoding{UTF-8}
    %\VignetteIndexEntry{CIMICE-R: (Markov) Chain Method to Infer Cancer Evolution}
    %\VignetteEngine{knitr::rmarkdown}
---

```{r style, echo = FALSE, results = 'asis'}
BiocStyle::markdown()
```

```{r, echo=F}
#knitr::opts_chunk$set(echo = TRUE, fig.align = "left")
```

```{r, include = FALSE}
show_matrix <- function(M, w, h){
    M[1:h,1:w] %>% as.matrix %>%  as.data.frame
}
size_matrix <- function(df){
    paste("ncol:", ncol(df), " - nrow:", nrow(df))
}
```


# Introduction 

This document is a presentation of the R implementation of the tool [CIMICE](https://github.com/redsnic/tumorEvolutionWithMarkovChains).
It shows the main features of this software and how it is built as a modular pipeline, with the goal of making it easy to change and update.

CIMICE is a tool in the field of tumor phylogenetics and
its goal is to build a Markov Chain (called Cancer Progression Markov Chain, CPMC) in order to model tumor subtypes evolution.
The input of CIMICE is a Mutational Matrix, so a boolean matrix representing altered genes in a
collection of samples. These samples are assumed to be obtained with single-cell DNA analysis techniques and
the tool is specifically written to use the peculiarities of this data for the CMPC construction.

CIMICE data processing and analysis can be divided in four section:

* Input management
* Graph topology reconstruction
* Graph weight computation
* Output presentation

These steps will be presented in the following sections.

## Used libraries 

This implementation of CIMICE is built as a single library on its own:

```{r, error=F, message=F, results = "hide"}
library(CIMICE)
```

and it requires the following libraries: 

`r Biocpkg("IRanges")`

```{r, error=F, message=F, results = "hide"}
# Dataframe manipulation
library(dplyr) 
# Plot display
library(ggplot2)
# Improved string operations
library(glue)
# Dataframe manipulation
library(tidyr)
# Graph data management
library(igraph)
# Remove transitive edges on a graph
# library(relations)
# Interactive graph visualization
library(networkD3)
# Interactive graph visualization
library(visNetwork)
# Correlation plot visualization
library(ggcorrplot)
# Functional R programming
library(purrr)
# Graph Visualization
library(ggraph)
# sparse matrices
library(Matrix)
```

# Input management

CIMICE requires a boolean dataframe as input, structured as follows:

* Each column represents a gene
* Each row represents a sample (or a genotype)
* Each 0/1 represents if a given gene is mutated in a given sample 

It is possible to load this information from a file. The default input format for CIMICE is the "CAPRI/CAPRESE" `r Biocpkg("TRONCO")` format:

* The file is a tab or space separated file
* The first line starts with the string "s\\g" (or any other word) followed by the list of genes (or loci) to be considered in the analysis
* Each subsequent line starts with a sample identifier string, followed by the bit set representing its genotype 

This is a scheme of CIMICE's input format:

```
s\g       gene_1 gene_2 ... gene_n
sample_1    1      0    ...   0
...
sample_m    1      1    ...   1
```

and this an example on how to load a dataset from the file system:

```{r, results = 'hide'}
# Read input dataset in CAPRI/CAPRESE format
dataset.big <- read_CAPRI(system.file("extdata", "example.CAPRI", package = "CIMICE", mustWork = TRUE))
```


```{r, echo=F}
dataset.big %>% show_matrix(6,6) 
dataset.big %>% size_matrix()
```

Another option is to define directly the dataframe in R. This is made easy by the functions `make_dataset` and `update_df`, used as follows:

```{r}
# genes
dataset <- make_dataset(A,B,C,D) %>%
    # samples
    update_df("S1", 0, 0, 0, 1) %>%
    update_df("S2", 1, 0, 0, 0) %>%
    update_df("S3", 1, 0, 0, 0) %>%
    update_df("S4", 1, 0, 0, 1) %>%
    update_df("S5", 1, 1, 0, 1) %>%
    update_df("S6", 1, 1, 0, 1) %>%
    update_df("S7", 1, 0, 1, 1) %>%
    update_df("S8", 1, 1, 0, 1) 
```

with the following outcome:

```{r, echo=FALSE}
dataset
```

In the case your data is composed by samples with associated frequencies it is possible to use an alternative format
that we call "CAPRIpop":

```
s/g    gene_1 gene_2 ... gene_n freq
sample_1 1 0 ... 0 freq_s1
...
sample_m 1 1 ... 1 freq_sm
```

where the `freq` column is mandatory and sample must not be repeated. Frequencies
in the `freq` column will be automatically normalized. This format is meant
to be used with the functions `quick_run(dataset, mode="CAPRIpop")` for the
full analysis and `dataset_preprocessing_population(...)` for the preprocessing
stage only. The subsequent operations remain otherwise equal to those 
of the default format.

Another option is to compute a mutational matrix directly from a MAF file, which 
can be done as follows:

```{r}
#        path to MAF file
read_MAF(system.file("extdata", "paac_jhu_2014_500.maf", package = "CIMICE", mustWork = TRUE))[1:5,1:5]
```

# Preliminary check of mutations distributions

This implementation of CIMICE includes simple functions to quickly analyze the distributions of mutations among genes and samples.

The following code displays an histogram showing the distribution of the number of mutations hitting a gene:

```{r}
gene_mutations_hist(dataset.big)
```

And this does the same but from the samples point of view:

```{r}
sample_mutations_hist(dataset.big, binwidth = 10)
```

## Simple procedures of feature selection

In case of huge dataset, it could be necessary to focus only on a subset of the input samples or genes.
The following procedures aim to provide an easy way to do so when the goal is to use the most (or least)
mutated samples or genes.

### By genes

Keeps the first $n$ (=100) most mutated genes: 

```{r, eval = FALSE}
select_genes_on_mutations(dataset.big, 100) 
```

```{r, echo = FALSE}
temp <- select_genes_on_mutations(dataset.big, 100)
temp %>% show_matrix(6,6) 
temp %>% size_matrix()
```

### By samples

Keeps the first $n$ (=100) least mutated samples: 

```{r, eval = FALSE}
select_samples_on_mutations(dataset.big, 100, desc = FALSE)
```

```{r, echo=FALSE}
temp <- select_samples_on_mutations(dataset.big, 100, desc = FALSE)
temp %>% show_matrix(6,6) 
temp %>% size_matrix()
```

## Both selections

It is easy to combine these selections by using the pipe operator `%>%`:

```{r, eval = FALSE}
select_samples_on_mutations(dataset.big , 100, desc = FALSE) %>% select_genes_on_mutations(100)
```
```{r, echo=FALSE}
temp <- select_samples_on_mutations(dataset.big , 100, desc = FALSE) %>% select_genes_on_mutations(100)
temp %>% show_matrix(6,6) 
temp %>% size_matrix()
```

## Correlation plot

It may be of interest to show correlations among gene or sample mutations. The library `corrplots` provides an easy way to do so by preparing an 
heatmap based on the correlation matrix. We can show these plots by using the following comands:

gene mutations correlation:

```{r}
corrplot_genes(dataset)
```

sample mutations correlation:

```{r}
corrplot_samples(dataset)
```

## Group equal genotypes

The first step of the CIMICE algorithm is based on grouping the genotypes contained in the dataset to compute their observed frequencies.
In this implementation we used a simple approach using the library `dplyr`. However, this solution is not optimal from an efficiency
point of view and might be problematic with very large datasets. An Rcpp implementation is planned and, moreover, it is possible to easily modify 
this step by changing the algorithm as long as its output is a dataframe containing only unique genotypes with an additional column named "freq" for the observed frequencies count.

```{r}
# groups and counts equal genotypes
compactedDataset <- compact_dataset(dataset)
```
```{r, echo = FALSE}
compactedDataset
```

# Graph topology construction

The subsequent stage goal is to prepare the topology for the final Cancer Progression Markov Chain. We racall that this topology is assumed to be 
a DAG. These eraly steps are required to prepare the information necessary for this and the following pahses.

Convert dataset to matricial form:

```{r}
samples <- compactedDataset$matrix
```
```{r,echo=F}
samples
```

Extract gene names:

```{r}
genes <- colnames(samples)
```
```{r,echo=F}
genes
```

Compute observed frequency of each genotype:

```{r}
freqs <- compactedDataset$counts/sum(compactedDataset$counts)
```
```{r, echo = FALSE}
freqs
```

Add "Clonal" genotype to the dataset (if not present) that will be used as DAG root:

```{r}
# prepare node labels listing the mutated genes for each node
labels <- prepare_labels(samples, genes)
if( is.null(compactedDataset$row_names) ){
    compactedDataset$row_names <- rownames(compactedDataset$matrix)
}
matching_samples <- compactedDataset$row_names
# fix Colonal genotype absence, if needed
fix <- fix_clonal_genotype(samples, freqs, labels, matching_samples)
samples = fix[["samples"]]
freqs = fix[["freqs"]]
labels = fix[["labels"]]
matching_samples <- fix[["matching_samples"]]
```
```{r,echo=F}
samples
```

Build the topology of the graph based on the "superset" relation: 

```{r}
# compute edges based on subset relation
edges <- build_topology_subset(samples)
```

and finally prepare and show with the current topology of the graph:

```{r}
# remove transitive edges and prepare igraph object
g <- build_subset_graph(edges, labels)
```

that can be (badly) plotted using basic igraph:

```{r, echo=F, out.height="300px",dpi=300}
V(g)$vertex.size <- rep(10, length(V(g)))
plot(g, vertex.size=rep(55, length(V(g))))
```

# Graph weight computation

In this sections, it is shown how to call the procedures to the four steps weight computation used in CIMICE. This is in fact based in computing 
"UP" weights, normalized "UP" weights, "DOWN" weights and normalized "DOWN" weights.

The process is based on the graph adjacency matrix "A":

```{r}
A <- as_adj(g)
```
```{r, echo=F}
A
```

and on the number of successors for each node:

```{r}
no.of.children <- get_no_of_children(A,g)
```
```{r, echo=F}
no.of.children
```

### "UP" weights

$$W_{up}(\langle a,b \rangle \in E) = \frac{1}{ |\Lambda_a|}(P(a) + \sum_{x\in \Pi_a}W_{up}(\langle x,a \rangle)) $$
given that $\Lambda_a$ and $\Pi_a$ denote the set of children of a node $a$ and the set of parents of a node $a$ respectively and that $P(a)$ is the observed frequency of node $a$.

```{r}
upWeights <- computeUPW(g, freqs, no.of.children, A)
```
```{r, echo=F}
upWeights
```

### "UP" weights normalization

$$\overline{W}_{up}(\langle a,b \rangle \in E_1) = \begin{cases}
                                                            1 & \mbox{if $a[0]=\emptyset$} \\
                                                            \frac{ W_{up}(\langle a,b \rangle \in E)}{\sum_{x \in \Pi_b} W_{up}(\langle x,b \rangle)} & \mbox{else} 
                                                \end{cases}$$   

```{r}
normUpWeights <- normalizeUPW(g, freqs, no.of.children, A, upWeights)
```
```{r, echo=F}
normUpWeights
```

### "DOWN" Weights

$$W_{down}(\langle a,b \rangle) = \overline{W}_{up}(\langle a,b \rangle)(P(b) + \sum_{x\in \Lambda_b}W_{down}(\langle b,x \rangle))$$


```{r}
downWeights <- computeDWNW(g, freqs, no.of.children, A, normUpWeights)
```
```{r, echo=F}
downWeights
```

### "DOWN" weights normalization

$$ P(\langle a,b \rangle) = \overline{W}_{down}(\langle a,b \rangle) = \frac{W_{down}(\langle a,b \rangle)}{\sum_{x\in \Lambda_a}{W_{down}(\langle a,x \rangle)}} $$

```{r}
normDownWeights <- normalizeDWNW(g, freqs, no.of.children, A, downWeights)
```
```{r, echo=F}
normDownWeights
```

# Output presentation

To better show the results of the analysis there were prepared three ouput methods based on three different libraries: `ggraph`, `networkD3` and 
`visNetwork`. These libraries improve the dafault `igraph` output visualization. Note that output interaction is disabled in this document,
check the Quick Guide instead.

This is the output based on `ggraph`, it is ideal for small graphs but, for legibility reasons, it is better not to use it with long labels.

```{r}
draw_ggraph(quick_run(example_dataset()))
```

The `networkD3` is a quite valid interactive approach, but it lacks the option to draw labels on edges, limiting the representation to thicker or thinner edges.

```{r, results = 'hide'}
draw_networkD3(quick_run(example_dataset()))
```

```{r, echo=FALSE, out.width = "100%", out.height="100%"}
knitr::include_graphics("networkD3.png")
```


The `visNetwork` approach is overall the best for interactive purposes. It allows almost arbitrary long labels, as it is compatible with `HTML` elements and
in particular with textboxes and the "hovering condition" for vertex and edges.

```{r, results = 'hide'}
draw_visNetwork(quick_run(example_dataset()))
```
```{r, echo=FALSE, out.width = "100%", out.height="100%"}
knitr::include_graphics("visGraph.png")
```

Finally, it is also possible to export CIMICE's output to the standard [dot format](https://en.wikipedia.org/wiki/DOT_%28graph_description_language%29)
for use in other visualization applications.

```{r}
cat(to_dot(quick_run(example_dataset())))
```


## Session information

This vignette was prepared using a R session with the following specifications:

```{r}
sessionInfo()
```


```{r, echo=FALSE}
# run ALL
```

---
title: "Quick guide"
author: "Nicolò Rossi"
date: "28/3/2020"
output: 
    html_document:
        df_print: paged
vignette: >
    %\VignetteIndexEntry{Quick guide}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(CIMICE)
library(dplyr)
library(igraph)
library(Matrix)
library(purrr)
```

## Reference for quick CIMICE analysis

In this short tutorial it is shown how to execute CIMICE's analysis

### Read input

To load the input use `CIMICE::read` family functions or the pair `CIMICE::make_dataset`, `CIMICE::update_df` in the following manner: 

```{r cars, results=FALSE}
# read from file
read_CAPRI(system.file("extdata", "example.CAPRI", package = "CIMICE", mustWork = TRUE))
```

```{r}
# from a string
read_CAPRI_string("
s\\g A B C D
S1 0 0 0 1
S2 1 0 0 0
S3 1 0 0 0
S4 1 0 0 1
S5 1 1 0 1
S6 1 1 0 1
S7 1 0 1 1
S8 1 1 0 1
")
```

```{r}
# using CIMICE::make_dataset and CIMICE::update_df
# genes
make_dataset(A,B,C,D) %>% 
    # samples
    update_df("S1", 0, 0, 0, 1) %>%
    update_df("S2", 1, 0, 0, 0) %>%
    update_df("S3", 1, 0, 0, 0) %>%
    update_df("S4", 1, 0, 0, 1) %>%
    update_df("S5", 1, 1, 0, 1) %>%
    update_df("S6", 1, 1, 0, 1) %>%
    update_df("S7", 1, 0, 1, 1) %>%
    update_df("S8", 1, 1, 0, 1)
```

This last dataset will be use in this example, under the name of `example_dataset()`. Input dataset analysis and feature selection should be done at this point.

## Preprocess dataset

This early phase reorganizes the dataset to simplify further analysis.

```{r}
preproc <- dataset_preprocessing(example_dataset())
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
```

## Build topology

The default method to organize genotypes in a graph in CIMICE is based on the "subset" relation among them.

```{r}
g <- graph_non_transitive_subset_topology(samples,labels)
```

```{r, echo=FALSE}
V(g)$vertex.size <- rep(10, length(V(g)))
plot(g, vertex.size=rep(55, length(V(g))))
```

## Compute Weights

CIMICE provides a strategy to estimate transition probabilities among genotypes. This phase is divided in four different subphases that can be executed together
in this way:

```{r}
W <- compute_weights_default(g, freqs)
```

## Visualize output:

CIMICE supports many different output options, the recommended one is based on "visNetwork".

```{r}
out <- quick_run(example_dataset()) # quick_run summarizes all the previous steps
draw_visNetwork(out)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{prepare_labels}
\alias{prepare_labels}
\title{Prepare node labels based on genotypes}
\usage{
prepare_labels(samples, genes)
}
\arguments{
\item{samples}{input dataset (mutational matrix) as matrix}

\item{genes}{list of gene names (in the columns' order)}
}
\value{
the computed edge list
}
\description{
Prepare node labels so that each node is labelled with a
comma separated list of the alterated genes
representing its associated genotype.
}
\details{
Note that after this procedure the user is
expected also to run fix_clonal_genotype
to also add the clonal genortype to the
mutational matrix if it is not present.
}
\examples{
require(dplyr) 

# compact
compactedDataset <- compact_dataset(example_dataset())
samples <- compactedDataset$matrix

# save genes' names
genes <- colnames(compactedDataset$matrix)

# keep the information on frequencies for further analysis
freqs <- compactedDataset$counts/sum(compactedDataset$counts)

# prepare node labels listing the mutated genes for each node
labels <- prepare_labels(samples, genes)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.R
\name{normalizeUPW}
\alias{normalizeUPW}
\title{Up weights normalization}
\usage{
normalizeUPW(g, freqs, no.of.children, A, upWeights)
}
\arguments{
\item{g}{graph (a Directed Acyclic Graph)}

\item{freqs}{observed genotype frequencies}

\item{no.of.children}{number of children for each node}

\item{A}{adjacency matrix of G}

\item{upWeights}{Up weights as computed by computeUPW}
}
\value{
a vector containing the normalized Up weights for each edge
}
\description{
Normalizes up weights so that the sum
of weights of edges entering in a node is 1
}
\examples{
require(dplyr)
require(igraph)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
g <- graph_non_transitive_subset_topology(samples, labels)
# prepare adj matrix
A <- as.matrix(as_adj(g))
# pre-compute exiting edges from each node
no.of.children <- get_no_of_children(A,g)
upWeights <- computeUPW(g, freqs, no.of.children, A)
normalizeUPW(g, freqs, no.of.children, A, upWeights)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{draw_ggraph}
\alias{draw_ggraph}
\title{ggplot graph output}
\usage{
draw_ggraph(out, digits = 4, ...)
}
\arguments{
\item{out}{the output object of CIMICE (es, from quick run)}

\item{digits}{precision for edges' weights}

\item{...}{other arguments for format_labels}
}
\value{
ggraph object representing g as described
}
\description{
Draws the output graph using ggplot
}
\examples{
draw_ggraph(quick_run(example_dataset()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{perturb_dataset}
\alias{perturb_dataset}
\title{Perturbate a boolean matrix}
\usage{
perturb_dataset(dataset, FP_rate = 0, FN_rate = 0, MIS_rate = 0)
}
\arguments{
\item{dataset}{a matrix/sparse matrix}

\item{FP_rate}{False Positive rate}

\item{FN_rate}{False Negative rate}

\item{MIS_rate}{Missing Data rate}
}
\value{
the new, perturbed, matrix
}
\description{
Given a boolean matrix, randomly 
add False Positives (FP), False Negatives (FN) and Missing data 
following user defined rates. In the final matrix, missing 
data is represented by the value 3.
}
\details{
Note that CIMICE does not support dataset with missing data natively, 
so using MIS_rate != 0 will then require some pre-processing.
}
\examples{
require(dplyr)

example_dataset() \%>\%
  make_generator_stub() \%>\% 
  set_generator_edges(
    list(
      "D", "A, D", 1 , 
      "A", "A, D", 1 , 
      "A, D", "A, C, D", 1 , 
      "A, D", "A, B, D", 1 , 
      "Clonal", "D", 1 , 
      "Clonal", "A", 1 , 
      "D", "D", 1 , 
      "A", "A", 1 , 
      "A, D", "A, D", 1 , 
      "A, C, D", "A, C, D", 1 , 
      "A, B, D", "A, B, D", 1 , 
      "Clonal", "Clonal", 1 
  )) \%>\% 
  finalize_generator \%>\% 
  simulate_generator(3, 10) \%>\% 
  perturb_dataset(FP_rate = 0.01, FN_rate = 0.1, MIS_rate = 0.12)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{graph_non_transitive_subset_topology}
\alias{graph_non_transitive_subset_topology}
\title{Default preparation of graph topology}
\usage{
graph_non_transitive_subset_topology(samples, labels)
}
\arguments{
\item{samples}{mutational matrix}

\item{labels}{genotype labels}
}
\value{
a graph with the wanted topology
}
\description{
By default, CIMICE computes the relation between
genotypes using the subset relation.
For the following steps it is also important
that the transitive edges are removed.
}
\examples{
require(dplyr)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
graph_non_transitive_subset_topology(samples, labels)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{annotate_mutational_matrix}
\alias{annotate_mutational_matrix}
\title{Add samples and genes names to a mutational matrix}
\usage{
annotate_mutational_matrix(M, samples, genes)
}
\arguments{
\item{M}{mutational matrix}

\item{samples}{list of sample names}

\item{genes}{list of gene names}
}
\value{
N with the set row and column names
}
\description{
Given M mutational matrix, add samples as row names, and genes as 
column names. If there are repetitions in row names, these are
solved by adding a sequential identifier to the names.
}
\examples{
require(Matrix)
genes <- c("A", "B", "C")
samples <- c("S1", "S2", "S2")
M <- Matrix(c(0,0,1,0,0,1,0,1,1), ncol=3, sparse=TRUE, byrow = TRUE)

annotate_mutational_matrix(M, samples, genes)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.R
\name{computeDWNW}
\alias{computeDWNW}
\title{Down weights computation}
\usage{
computeDWNW(g, freqs, no.of.children, A, normUpWeights)
}
\arguments{
\item{g}{graph (a Directed Acyclic Graph)}

\item{freqs}{observed genotype frequencies}

\item{no.of.children}{number of children for each node}

\item{A}{adjacency matrix of G}

\item{normUpWeights}{normalized up weights as computed by normalizeUPW}
}
\value{
a vector containing the Up weights for each edge
}
\description{
Computes the Down weights formula using
a Dinamic Programming approach (starting call),
see vignettes for further explaination.
}
\examples{
require(dplyr)
require(igraph)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
g <- graph_non_transitive_subset_topology(samples, labels)
# prepare adj matrix
A <- as.matrix(as_adj(g))
# pre-compute exiting edges from each node
no.of.children <- get_no_of_children(A,g)
upWeights <- computeUPW(g, freqs, no.of.children, A)
normUpWeights <- normalizeUPW(g, freqs, no.of.children, A, upWeights)
computeDWNW(g, freqs, no.of.children, A, normUpWeights)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{select_genes_on_mutations}
\alias{select_genes_on_mutations}
\title{Filter dataset by genes' mutation count}
\usage{
select_genes_on_mutations(mutmatrix, n, desc = TRUE)
}
\arguments{
\item{mutmatrix}{input dataset (mutational matrix) to be reduced}

\item{n}{number of genes to be kept}

\item{desc}{TRUE: select the n least mutated genes,
FALSE: select the n most mutated genes}
}
\value{
the modified dataset (mutational matrix)
}
\description{
Dataset filtering on genes, based on their mutation count
}
\examples{

# keep information on the 100 most mutated genes
select_genes_on_mutations(example_dataset(), 5)
# keep information on the 100 least mutated genes
select_genes_on_mutations(example_dataset(), 5, desc = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{select_samples_on_mutations}
\alias{select_samples_on_mutations}
\title{Filter dataset by samples' mutation count}
\usage{
select_samples_on_mutations(mutmatrix, n, desc = TRUE)
}
\arguments{
\item{mutmatrix}{input dataset (mutational matrix) to be reduced}

\item{n}{number of samples to be kept}

\item{desc}{T: select the n least mutated samples,
F: select the n most mutated samples}
}
\value{
the modified dataset (mutational matrix)
}
\description{
Dataset filtering on samples, based on their mutation count
}
\examples{
require(dplyr)
# keep information on the 5 most mutated samples
select_samples_on_mutations(example_dataset(), 5)
# keep information on the 5 least mutated samples
select_samples_on_mutations(example_dataset(), 5, desc = FALSE)
# combine selections
select_samples_on_mutations(example_dataset() , 5, desc = FALSE) \%>\%
    select_genes_on_mutations(5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{example_dataset}
\alias{example_dataset}
\title{Creates a simple example dataset}
\usage{
example_dataset()
}
\value{
a simple mutational matrix
}
\description{
Creates a simple example dataset
}
\examples{
example_dataset()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{dataset_preprocessing}
\alias{dataset_preprocessing}
\title{Run CIMICE preprocessing}
\usage{
dataset_preprocessing(dataset)
}
\arguments{
\item{dataset}{a mutational matrix as a (sparse) matrix}
}
\value{
a list containing the mutational matrix ("samples"),
the mutational frequencies of the genotypes ("freqs"),
the node labels ("labels") and finally the gene names ("genes")
}
\description{
executes the preprocessing steps of CIMICE
}
\details{
Preprocessing steps:

1) dataset is compacted

2) genotype frequencies are computed

3) labels are prepared
}
\examples{
require(dplyr)
example_dataset() \%>\% dataset_preprocessing

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{fix_clonal_genotype}
\alias{fix_clonal_genotype}
\title{Manage Clonal genotype in data}
\usage{
fix_clonal_genotype(samples, freqs, labels, matching_samples)
}
\arguments{
\item{samples}{input dataset (mutational matrix) as matrix}

\item{freqs}{genotype frequencies (in the rows' order)}

\item{labels}{list of gene names (in the columns' order)}

\item{matching_samples}{list of sample names matching each genotype}
}
\value{
a named list containing the fixed "samples", "freqs" and "labels"
}
\description{
Fix the absence of the clonal genotype in the data (if needed)
}
\examples{
require(dplyr) 

# compact
compactedDataset <- compact_dataset(example_dataset())
samples <- compactedDataset$matrix

# save genes' names
genes <- colnames(compactedDataset$matrix)

# keep the information on frequencies for further analysis
freqs <- compactedDataset$counts/sum(compactedDataset$counts)

# prepare node labels listing the mutated genes for each node
labels <- prepare_labels(samples, genes)
if( is.null(compactedDataset$row_names) ){
  compactedDataset$row_names <- rownames(compactedDataset$matrix)
}
matching_samples <- compactedDataset$row_names
# matching_samples
matching_samples 

# fix Colonal genotype absence, if needed
fix <- fix_clonal_genotype(samples, freqs, labels, matching_samples)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_matrix}
\alias{read_matrix}
\title{Read dataset from an R matrix}
\usage{
read_matrix(mat)
}
\arguments{
\item{mat}{a boolean mutational matrix}
}
\value{
the sparse mutational matrix to be used as CIMICE's input
}
\description{
also converts that matrix to a sparse matrix
}
\examples{
m <- matrix(c(0,0,1,1,0,1,1,1,1), ncol=3)
colnames(m) <- c("A","B","C")
rownames(m) <- c("S1", "S2", "S3")
read_matrix(m)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{compute_weights_default}
\alias{compute_weights_default}
\title{Compute default weights}
\usage{
compute_weights_default(g, freqs)
}
\arguments{
\item{g}{a graph (must be a DAG with no transitive edges)}

\item{freqs}{observed frequencies of genotypes}
}
\value{
a graph with the computed weights
}
\description{
This procedure computes the weights for edges of a
graph accordingly to CIMICE specification.
(See vignettes for further explainations)
}
\examples{
require(dplyr)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
g <- graph_non_transitive_subset_topology(samples, labels)
compute_weights_default(g, freqs)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{prepare_generator_edge_set_command}
\alias{prepare_generator_edge_set_command}
\title{Prepare a command to add edge weights to a generator}
\usage{
prepare_generator_edge_set_command(generator, by = "labels")
}
\arguments{
\item{generator}{a generator}

\item{by}{"labels" or "samples" to use gene labels or sample labels as references for
edge identifiers.}
}
\value{
NULL (the string with the function calls is printed on the stdout)
}
\description{
Prints a string in the form of the command that 
sets weights for all the edges of this generator.
}
\examples{
require(dplyr)
example_dataset() \%>\% 
  make_generator_stub() \%>\%
  prepare_generator_edge_set_command()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_MAF}
\alias{read_MAF}
\title{Create mutational matrix from MAF file}
\usage{
read_MAF(path, ...)
}
\arguments{
\item{path}{path to MAF file}

\item{...}{other maftools::mutCountMatrix arguments}
}
\value{
the mutational (sparse) matrix associated to the MAF file
}
\description{
Read a MAF (Mutation Annotation Format) file and create
a Mutational Matrix combining gene and sample IDs.
}
\examples{
read_MAF(system.file("extdata", "paac_jhu_2014_500.maf", package = "CIMICE", mustWork = TRUE))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_CAPRI_string}
\alias{read_CAPRI_string}
\title{Read "CAPRI" file from a string}
\usage{
read_CAPRI_string(txt)
}
\arguments{
\item{txt}{string in valid "CAPRI" format}
}
\value{
the described mutational matrix as a (sparse) matrix
}
\description{
Read a "CAPRI" formatted file, from a text string
}
\examples{
read_CAPRI_string("
s\\\\g A B C D
S1 0 0 0 1
S2 1 0 0 0
S3 1 0 0 0
S4 1 0 0 1
S5 1 1 0 1
S6 1 1 0 1
S7 1 0 1 1
S8 1 1 0 1
")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{remove_transitive_edges}
\alias{remove_transitive_edges}
\title{Remove transitive edges from an edgelist}
\usage{
remove_transitive_edges(E)
}
\arguments{
\item{E}{edge list, built from "build_topology_subset"}
}
\value{
a new edgelist without transitive edges (as a N*2 matrix)
}
\description{
Remove transitive edges from an edgelist. This procedure is temporary to
cover a bug in 'relations' package.
}
\examples{
l <- list(c(1,2),c(2,3), c(1,3))
remove_transitive_edges(l)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{simulate_generator}
\alias{simulate_generator}
\title{Create datasets from generators}
\usage{
simulate_generator(
  generator,
  time_ticks,
  times,
  starting_label = "Clonal",
  by = "labels",
  mode = "full"
)
}
\arguments{
\item{generator}{a generator}

\item{time_ticks}{number of steps (updates) of the DTMC associated to the generato}

\item{times}{number of sumlated cells}

\item{starting_label}{node from which to start the simulation}

\item{by}{"labels" or "samples" to use gene labels or sample labels as references to identify the `starting_label`'s node}

\item{mode}{"full" to generate a matrix with `times` genotypes, "compacted" to *efficiently* create an already compacted dataset 
(a dataset showing the genotypes and their respective frequencies)}
}
\value{
the simulated dataset
}
\description{
Simulate the DTMC associated to the generator to create a dataset
that reflects the genotypes of `times` cells, sampled after
`time_ticks` passages.
}
\examples{
require(dplyr)

example_dataset() \%>\%
  make_generator_stub() \%>\% 
  set_generator_edges(
    list(
      "D", "A, D", 1 , 
      "A", "A, D", 1 , 
      "A, D", "A, C, D", 1 , 
      "A, D", "A, B, D", 1 , 
      "Clonal", "D", 1 , 
      "Clonal", "A", 1 , 
      "D", "D", 1 , 
      "A", "A", 1 , 
      "A, D", "A, D", 1 , 
      "A, C, D", "A, C, D", 1 , 
      "A, B, D", "A, B, D", 1 , 
      "Clonal", "Clonal", 1 
  )) \%>\% 
  finalize_generator \%>\% 
  simulate_generator(3, 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVisualization.R
\name{gene_mutations_hist}
\alias{gene_mutations_hist}
\title{Histogram of genes' frequencies}
\usage{
gene_mutations_hist(mutmatrix, binwidth = 1)
}
\arguments{
\item{mutmatrix}{input dataset (mutational matrix)}

\item{binwidth}{binwidth parameter for the histogram (as in ggplot)}
}
\value{
the newly created histogram
}
\description{
Create the histogram of the genes' mutational frequencies
}
\examples{
gene_mutations_hist(example_dataset(), binwidth = 10)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{format_labels}
\alias{format_labels}
\title{Format labels for output object}
\usage{
format_labels(labels, max_col = 3, max_row = 3)
}
\arguments{
\item{labels}{a charachter vector of the labels to manage}

\item{max_col}{maximum number of identifiers in a single row for a label}

\item{max_row}{maximum number of rows of identifiers in a label}
}
\value{
the updated labels
}
\description{
Prepare labels based on multiple identifiers so that they do not excede
a certain size (if they do, a simple number is used)
}
\examples{
 
format_labels(c("A, B", "C, D, E"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{finalize_generator}
\alias{finalize_generator}
\title{Finalize generator normalizing edge weights}
\usage{
finalize_generator(generator)
}
\arguments{
\item{generator}{a generator}
}
\value{
A generator with edge weights that respect DTMC definition
}
\description{
Checks if a generator can be normalized so that it 
actually is a Markov Chain
}
\examples{
require(dplyr)

example_dataset() \%>\%
 make_generator_stub() \%>\% 
 set_generator_edges(
   list(
    "D", "A, D", 1 , 
    "A", "A, D", 1 , 
    "A, D", "A, C, D", 1 , 
    "A, D", "A, B, D", 1 , 
    "Clonal", "D", 1 , 
    "Clonal", "A", 1 , 
    "D", "D", 1 , 
    "A", "A", 1 , 
    "A, D", "A, D", 1 , 
    "A, C, D", "A, C, D", 1 , 
    "A, B, D", "A, B, D", 1 , 
    "Clonal", "Clonal", 1 
  )) \%>\% 
  finalize_generator

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{build_topology_subset}
\alias{build_topology_subset}
\title{Compute subset relation as edge list}
\usage{
build_topology_subset(samples)
}
\arguments{
\item{samples}{input dataset (mutational matrix) as matrix}
}
\value{
the computed edge list
}
\description{
Create an edge list E representing the
'subset' relation for binary strings so that:
\deqn{ (A,B) in E <=> forall(i) : A[i] -> B[i] }
}
\examples{
require(dplyr)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
build_topology_subset(samples)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_CAPRIpop_string}
\alias{read_CAPRIpop_string}
\title{Read "CAPRIpop" file from a string}
\usage{
read_CAPRIpop_string(txt)
}
\arguments{
\item{txt}{string in valid "CAPRIpop" format}
}
\value{
the described mutational matrix as a (sparse) matrix
}
\description{
Read a "CAPRIpop" formatted file, from a text string
}
\examples{
read_CAPRIpop_string("
s\\\\g A B C D freqs
S1 0 0 0 1 0.1
S2 1 0 0 0 0.1
S3 1 0 0 0 0.2
S4 1 0 0 1 0.3
S5 1 1 0 1 0.05
S6 1 1 0 1 0.1
S7 1 0 1 1 0.05
S8 1 1 0 1 0.01
")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CIMICE-R.R
\docType{package}
\name{CIMICE}
\alias{CIMICE}
\title{CIMICE Package}
\description{
R implementation of the CIMICE tool.
CIMICE is a tool in the field of tumor phylogenetics and
its goal is to build a Markov Chain (called Cancer Progression Markov Chain, CPMC) in order to model tumor subtypes evolution.
The input of CIMICE is a Mutational Matrix, so a boolean matrix representing altered genes in a
collection of samples. These samples are assumed to be obtained with single-cell DNA analysis techniques and
the tool is specifically written to use the peculiarities of this data for the CMPC construction.
See `https://github.com/redsnic/tumorEvolutionWithMarkovChains/tree/master/GenotypeEvolutionPaths` for the
original Java version of this tool.
}
\details{
CIMICE-R: (Markov) Chain Method to Infer Cancer Evolution
}
\author{
Nicolò Rossi \email{olocin.issor@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{example_dataset_withFreqs}
\alias{example_dataset_withFreqs}
\title{Creates a simple example dataset with frequency column}
\usage{
example_dataset_withFreqs()
}
\value{
a simple mutational matrix
}
\description{
Creates a simple example dataset with frequency column
}
\examples{
example_dataset_withFreqs()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_CAPRI}
\alias{read_CAPRI}
\title{Read a "CAPRI" file}
\usage{
read_CAPRI(filepath)
}
\arguments{
\item{filepath}{path to file}
}
\value{
the described mutational matrix as a (sparse) matrix
}
\description{
Read a "CAPRI" formatted file from the file system
}
\examples{
#          "pathToDataset/myDataset.CAPRI"
read_CAPRI(system.file("extdata", "example.CAPRI", package = "CIMICE", mustWork = TRUE))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{chunk_reader}
\alias{chunk_reader}
\title{Gradually read a file from disk}
\usage{
chunk_reader(file_path)
}
\arguments{
\item{file_path}{path to large file}
}
\value{
a list-object containing the function `read` to 
read lines from the given file, and `close` to close the
connection to the file stream.
}
\description{
This function creates a reader to read a text file in batches (or chunks). 
It can be used for very large files that cannot fit in RAM.
}
\examples{
# open connection to file
reader <- chunk_reader(
    system.file("extdata", "paac_jhu_2014_500.maf", package = "CIMICE", mustWork = TRUE)
)

while(TRUE){
    # read a chunk
    chunk <- reader$read(10)
    if(length(chunk) == 0){
        break
    }    
    # --- process chunk ---
}
# close connection
reader$close()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{quick_run}
\alias{quick_run}
\title{Run CIMICE defaults}
\usage{
quick_run(dataset, mode = "CAPRI")
}
\arguments{
\item{dataset}{a mutational matrix as a (sparse) matrix}

\item{mode}{indicates the used input format. Must be either "CAPRI" or "CAPRIpop"}
}
\value{
a list object representing the graph computed by CIMICE with the
structure `list(topology = g, weights = W, labels = labels, freqs=freqs)`
}
\description{
This function executes CIMICE analysis on a dataset using default settings.
}
\examples{
quick_run(example_dataset())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{get_no_of_children}
\alias{get_no_of_children}
\title{Get number of children}
\usage{
get_no_of_children(A, g)
}
\arguments{
\item{A}{Adjacency matrix of the graph g}

\item{g}{a graph}
}
\value{
a vector containing the number of children for each node in g
}
\description{
Compute number of children for each node given an adj matrix
}
\examples{
require(dplyr)
require(igraph)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
g <- graph_non_transitive_subset_topology(samples, labels)
A <- as_adj(g)
get_no_of_children(A, g)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{read_CAPRIpop}
\alias{read_CAPRIpop}
\title{Read a "CAPRIpop" file}
\usage{
read_CAPRIpop(filepath)
}
\arguments{
\item{filepath}{path to file}
}
\value{
a list containing the described mutational matrix as a (sparse) matrix
        and a list of the frequency of the genotypes
}
\description{
Read a "CAPRIpop" formatted file from the file system
}
\examples{
#          "pathToDataset/myDataset.CAPRI"
read_CAPRI(system.file("extdata", "example.CAPRIpop", package = "CIMICE", mustWork = TRUE))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVisualization.R
\name{corrplot_genes}
\alias{corrplot_genes}
\title{Gene based correlation plot}
\usage{
corrplot_genes(mutmatrix)
}
\arguments{
\item{mutmatrix}{input dataset (mutational matrix)}
}
\value{
the computed correlation plot
}
\description{
Prepare a correlation plot computed from genes' perspective
using a mutational matrix
}
\examples{
corrplot_genes(example_dataset())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis.R
\name{dataset_preprocessing_population}
\alias{dataset_preprocessing_population}
\title{Run CIMICE preprocessing for poulation format dataset}
\usage{
dataset_preprocessing_population(compactedDataset)
}
\arguments{
\item{compactedDataset}{a list (matrix: a mutational matrix, 
counts: number of samples with given genotype). 
"counts" is normalized automatically.}
}
\value{
a list containing the mutational matrix ("samples"),
the mutational frequencies of the genotypes ("freqs"),
the node labels ("labels") and finally the gene names ("genes")
}
\description{
executes the preprocessing steps of CIMICE
}
\details{
Preprocessing steps:

1) genotype frequencies are computed

2) labels are prepared
}
\examples{
require(dplyr)
example_dataset_withFreqs() \%>\% dataset_preprocessing_population

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{to_dot}
\alias{to_dot}
\title{Dot graph output}
\usage{
to_dot(out, ...)
}
\arguments{
\item{out}{the output object of CIMICE (es, from quick run)}

\item{...}{other arguments for format_labels}
}
\value{
a string representing the graph in dot format
}
\description{
Represents this graph in dot format (a textual output format)
}
\examples{
to_dot(quick_run(example_dataset()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{make_labels}
\alias{make_labels}
\title{Helper function to create labels}
\usage{
make_labels(out, mode = "samplesIDs", max_col = 3, max_row = 3)
}
\arguments{
\item{out}{the output object of CIMICE (es, from quick run)}

\item{mode}{which labels to print: samplesIDs (matching samples), sequentialIDs (just a sequential numer), geneIDs (genotype)}

\item{max_col}{identifiers are represented in a max_col times max_row grid (if the number of IDs exceeds, a the sequentialID number is used instead)}

\item{max_row}{identifiers are represented in a max_col times max_row grid (if the number of IDs exceeds, a the sequentialID number is used instead)}
}
\value{
the requested labels
}
\description{
This function helps creating labels for nodes with different information
}
\examples{
 
make_labels(quick_run(example_dataset()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{build_subset_graph}
\alias{build_subset_graph}
\title{Remove transitive edges and prepare graph}
\usage{
build_subset_graph(edges, labels)
}
\arguments{
\item{edges}{edge list, built from "build_topology_subset"}

\item{labels}{list of node labels, to be paired with the graph}
}
\value{
a graph with the subset topology, omitting transitive edges
}
\description{
Create a graph from the "build_topology_subset" edge list, so
that it respects the subset relation, omitting the transitive edges.
}
\examples{
require(dplyr)
preproc <- example_dataset() \%>\% dataset_preprocessing
samples <- preproc[["samples"]]
freqs   <- preproc[["freqs"]]
labels  <- preproc[["labels"]]
genes   <- preproc[["genes"]]
edges <- build_topology_subset(samples)
g <- build_subset_graph(edges, labels)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{binary_radix_sort}
\alias{binary_radix_sort}
\title{Radix sort for a binary matrix}
\usage{
binary_radix_sort(mat)
}
\arguments{
\item{mat}{a binary matrix (of 0 and 1)}
}
\value{
the sorted matrix
}
\description{
Sort the rows of a binary matrix in ascending order
}
\examples{
require(Matrix)
m <- Matrix(c(1,1,0,1,0,0,0,1,1), sparse = TRUE, ncol = 3)
binary_radix_sort(m)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{update_df}
\alias{update_df}
\title{Dataset line by line construction: add a sample}
\usage{
update_df(mutmatrix, sampleName, ...)
}
\arguments{
\item{mutmatrix}{an existing (sparse) matrix (mutational matrix)}

\item{sampleName}{the row (sample) name}

\item{...}{sample's genotype (0/1 numbers)}
}
\value{
the modified (sparse) matrix (mutational matrix)
}
\description{
Add a sample (a row) to an existing dataset.
This procedure is meant to be used with the "%>%" operator
}
\examples{

require(dplyr)
make_dataset(APC,P53,KRAS)   \%>\%
    update_df("S1", 1, 0, 1) \%>\%
    update_df("S2", 1, 1, 1)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{make_dataset}
\alias{make_dataset}
\title{Dataset line by line construction: initialization}
\usage{
make_dataset(...)
}
\arguments{
\item{...}{gene names (do not use '"', the input
is automatically converted to strings)}
}
\value{
a mutational matrix without samples structured as (sparse) matrix
}
\description{
Initialize a dataset for "line by line" creation
}
\examples{
make_dataset(APC,P53,KRAS)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{set_generator_edges}
\alias{set_generator_edges}
\title{Add edge weights to a generator}
\usage{
set_generator_edges(generator, f_t_w_list, by = "labels")
}
\arguments{
\item{generator}{a generator}

\item{f_t_w_list}{a list of triplets (from, to, list), the triplets must not 
be nested in the list. For example list("A","B",0.3, "B", "C", 0.2) is a valid
input.}

\item{by}{"labels" or "samples" to use gene labels or sample labels as references for
edge identifiers.}
}
\value{
the generator with the modified edges (invalid edges are ignored)
}
\description{
Add edge weights to a generator
}
\examples{
require(dplyr)

example_dataset() \%>\%
 make_generator_stub() \%>\% 
 set_generator_edges(
   list(
    "D", "A, D", 1 , 
    "A", "A, D", 1 , 
    "A, D", "A, C, D", 1 , 
    "A, D", "A, B, D", 1 , 
    "Clonal", "D", 1 , 
    "Clonal", "A", 1 , 
    "D", "D", 1 , 
    "A", "A", 1 , 
    "A, D", "A, D", 1 , 
    "A, C, D", "A, C, D", 1 , 
    "A, B, D", "A, B, D", 1 , 
    "Clonal", "Clonal", 1 
  ))
   
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.R
\name{computeDWNW_aux}
\alias{computeDWNW_aux}
\title{Down weights computation (aux)}
\usage{
computeDWNW_aux(g, edge, freqs, no.of.children, A, normUpWeights)
}
\arguments{
\item{g}{graph (a Directed Acyclic Graph)}

\item{edge}{the currently considered edge}

\item{freqs}{observed genotype frequencies}

\item{no.of.children}{number of children for each node}

\item{A}{adjacency matrix of G}

\item{normUpWeights}{normalized up weights as computed by normalizeUPW}
}
\value{
a vector containing the Up weights for each edge
}
\description{
Computes the Down weights formula using
a Dinamic Programming approach (recursion),
see vignettes for further explaination.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inputVisualization.R
\name{corrplot_from_mutational_matrix}
\alias{corrplot_from_mutational_matrix}
\title{Correlation plot from mutational matrix}
\usage{
corrplot_from_mutational_matrix(mutmatrix)
}
\arguments{
\item{mutmatrix}{input dataset}
}
\value{
the computed correlation plot
}
\description{
Prepare correlation plot based on a mutational matrix
}
\examples{
corrplot_from_mutational_matrix(example_dataset())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocessing.R
\name{compact_dataset}
\alias{compact_dataset}
\title{Compact dataset rows}
\usage{
compact_dataset(mutmatrix)
}
\arguments{
\item{mutmatrix}{input dataset (mutational matrix)}
}
\value{
a list with matrix (the compacted dataset (mutational matrix)), counts 
(frequencies of genotypes) and row_names (comma separated string of sample IDs) fields
}
\description{
Count duplicate rows
and compact the dataset (mutational). The column
'freq' will contain the counts for each row.
}
\examples{
compact_dataset(example_dataset())

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outputUtils.R
\name{draw_networkD3}
\alias{draw_networkD3}
\title{NetworkD3 graph output}
\usage{
draw_networkD3(out, ...)
}
\arguments{
\item{out}{the output object of CIMICE (es, from quick run)}

\item{...}{other arguments for format_labels}
}
\value{
networkD3 object representing g as described
}
\description{
Draws the output graph using networkD3
}
\examples{
draw_networkD3(quick_run(example_dataset()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DatasetGenerator.R
\name{make_generator_stub}
\alias{make_generator_stub}
\title{Create a stub for a generator}
\usage{
make_generator_stub(dataset)
}
\arguments{
\item{dataset}{A compacted CIMICE dataset}
}
\value{
a generator, with weight = 0 for all the edges
}
\description{
Create a generator topology directly from a dataset.
The topology will follow the subset relation.
}
\examples{
make_generator_stub(example_dataset())

}
